@@ -1,6 +1,13 @@
 #include <osg/Version>
 #include <osg/ArgumentParser>
 #include <osg/ApplicationUsage>
+
+#include <osg/Matrix>
+#include <osg/Plane>
+#include <osg/BoundingBox>
+#include <osg/BoundingSphere>
+
+
 #include <OpenThreads/Version>
 
 #include <set>
@@ -205,6 +212,13 @@ bool validName(const std::string& first)
     if (first=="TexMat(Matrix") return false;
     if (first=="GraphicsWindowX11") return false;
     if (first=="OperationThread") return false;
+    if (first=="SimpleViewer") return false;
+    if (first=="IndexFaceSets") return false;
+    if (first=="Quicktime") return false;
+    if (first=="SceneGraphBuilder") return false;
+    if (first=="LightPointNode") return false;
+    if (first=="GeometryTechnique") return false;
+    if (first=="GeoemtryTechnique") return false;
     return true;
 }
 
@@ -217,7 +231,9 @@ std::string typoCorrection(const std::string& name)
     if (name=="") return "";
 #endif
     if (name=="Martsz") return "Martz";
+    if (name=="Matz")  return "Martz";
     if (name=="Froenlich") return "Frhlich";
+    if (name=="Froechlich") return "Frhlich";
     if (name=="Froelich") return "Frhlich";    
     if (name=="Fruciel") return "Frauciel";
     if (name=="Hebelin") return "Herbelin";
@@ -237,6 +253,7 @@ std::string typoCorrection(const std::string& name)
     if (name=="Lugi") return "Luigi";
     if (name=="Daved") return "David";
     if (name=="Fred") return "Frederic";
+    if (name=="Drederic") return "Frederic";
     if (name=="Fredrick") return "Frederic";
     if (name=="Fredric") return "Frederic";
     if (name=="Garrat") return "Garret";
@@ -315,14 +332,56 @@ std::string typoCorrection(const std::string& name)
     if (name=="Skinnder") return "Skinner";
     if (name=="Mihair")  return "Mihai";
     if (name=="Mahai")  return "Mihai";
-    
-    if (name=="SimpleViewer") return "";
-    
+    if (name=="Wojiech" || name=="Wojchiech")  return "Wojciech";
+    if (name=="Leandowski" || name=="Lawandowski")  return "Lewandowski";
+    if (name=="Aderian")  return "Adrian";
+    if (name=="Callue")  return "Callu";
+    if (name=="Waldrom")  return "Waldron";
+    if (name=="Atr")  return "Art";
+    if (name=="Sylvan") return "Sylvain";
+    if (name=="Giatan") return "Gaitan";
+    if (name=="Melchoir") return "Melchior";
+    if (name=="Simmonsson") return "Simonsson";
+    if (name=="Sokolwsky") return "Sokolowsky";
+    if (name=="Cullu") return "Callu";
+    if (name=="Comporesi") return "Camporesi";
+    if (name=="Machler") return "Mchler";
+    if (name=="Maechler") return "Mchler";
     return name;
 }
 
 void nameCorrection(NamePair& name)
 {
+    if ((name.first=="Jose" || name.first=="Jos") && name.second=="Delport")
+    {
+        name.first = "J.P.";
+        name.second = "Delport";
+    }
+    if (name.first=="Franz" && name.second=="Melchior")
+    {
+        name.first = "Melchior";
+        name.second = "Franz";
+    }
+    if (name.first=="Glen" && name.second=="Waldon")
+    {
+        name.first = "Glenn";
+        name.second = "Waldron";
+    }
+    if (name.first=="Ralf" && name.second=="Karn")
+    {
+        name.first = "Ralf";
+        name.second = "Kern";
+    }
+    if (name.first=="Donny" && name.second=="Cipperly")
+    {
+        name.first = "Donald";
+        name.second = "Cipperly";
+    }
+    if (name.first=="Gino" && name.second=="")
+    {
+        name.first = "Gino";
+        name.second = "van den Bergen";
+    }
     if (name.first=="Radu" && name.second=="Mihai")
     {
         name.first = "Mihai";
@@ -669,9 +728,16 @@ int main( int argc, char **argv)
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
     arguments.getApplicationUsage()->addCommandLineOption("--entries","Print out number of entries into the ChangeLog file for each contributor.");
     arguments.getApplicationUsage()->addCommandLineOption("--version-number","Print out version number only");
-    arguments.getApplicationUsage()->addCommandLineOption("--soversion-number","Print out shared object version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--major-number","Print out major version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--minor-number","Print out minor version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--patch-number","Print out patch version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--so-number ","Print out shared object version number only");
     arguments.getApplicationUsage()->addCommandLineOption("--openthreads-version-number","Print out version number for OpenThreads only");
     arguments.getApplicationUsage()->addCommandLineOption("--openthreads-soversion-number","Print out shared object version number for OpenThreads only");
+    arguments.getApplicationUsage()->addCommandLineOption("Matrix::value_type","Print the value of Matrix::value_type");
+    arguments.getApplicationUsage()->addCommandLineOption("Plane::value_type","Print the value of Plane::value_type");
+    arguments.getApplicationUsage()->addCommandLineOption("BoundingSphere::value_type","Print the value of BoundingSphere::value_type");
+    arguments.getApplicationUsage()->addCommandLineOption("BoundingBox::value_type","Print the value of BoundingBox::value_type");
     arguments.getApplicationUsage()->addCommandLineOption("-r <file> or --read <file>","Read the ChangeLog to generate an estimated contributors list.");
 
     if (arguments.read("--version-number"))
@@ -698,7 +764,7 @@ int main( int argc, char **argv)
         return 0;
     }
     
-    if (arguments.read("--soversion-number"))
+    if (arguments.read("--soversion-number") || arguments.read("--so-number") )
     {
         std::cout<<osgGetSOVersion()<<std::endl;
         return 0;
@@ -729,13 +795,36 @@ int main( int argc, char **argv)
         std::cout<<OPENTHREADS_PATCH_VERSION<<std::endl;
         return 0;
     }
-    
 
     if (arguments.read("--openthreads-soversion-number"))
     {
         std::cout<<OpenThreadsGetSOVersion()<<std::endl;
         return 0;
     }
+
+
+    if (arguments.read("Matrix::value_type"))
+    {
+        std::cout<<((sizeof(osg::Matrix::value_type)==4)?"float":"double")<<std::endl;
+        return 0;
+    }
+
+    if (arguments.read("Plane::value_type"))
+    {
+        std::cout<<((sizeof(osg::Plane::value_type)==4)?"float":"double")<<std::endl;
+        return 0;
+    }
+
+    if (arguments.read("BoundingSphere::value_type"))
+    {
+        std::cout<<((sizeof(osg::BoundingSphere::value_type)==4)?"float":"double")<<std::endl;
+        return 0;
+    }
+    if (arguments.read("BoundingBox::value_type"))
+    {
+        std::cout<<((sizeof(osg::BoundingBox::value_type)==4)?"float":"double")<<std::endl;
+        return 0;
+    }
     
     std::cout<<osgGetLibraryName()<< " "<< osgGetVersion()<<std::endl<<std::endl;
 
