@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgcluster.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #ifdef USE_MEM_CHECK
 #include <mcheck.h>
 #endif
@@ -8,11 +26,14 @@
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
 
-#include <osgProducer/Viewer>
+#include <osgGA/TrackballManipulator>
+#include <osgViewer/Viewer>
 
 #include <osg/Quat>
 #include <osg/io_utils>
 
+#include <iostream>
+
 #if defined (WIN32) && !defined(__CYGWIN__)
 #include <winsock.h>
 #endif
@@ -47,9 +68,9 @@ class CameraPacket {
             matrix = _matrix * osg::Matrix::rotate(osg::DegreesToRadians(angle_offset),0.0f,1.0f,0.0f);
         }
         
-        void readEventQueue(osgProducer::Viewer& viewer);
+        void readEventQueue(osgViewer::Viewer& viewer);
         
-        void writeEventQueue(osgProducer::Viewer& viewer);
+        void writeEventQueue(osgViewer::Viewer& viewer);
 
         void setMasterKilled(const bool flag) { _masterKilled = flag; }
         const bool getMasterKilled() const { return _masterKilled; }
@@ -233,12 +254,14 @@ class DataConverter
 
             writeUInt(fs.getFrameNumber());
             writeDouble(fs.getReferenceTime());
+            writeDouble(fs.getSimulationTime());
         }
 
         void read(osg::FrameStamp& fs)
         {
             fs.setFrameNumber(readUInt());
             fs.setReferenceTime(readDouble());
+            fs.setSimulationTime(readDouble());
 
             osg::notify(osg::NOTICE)<<"readFramestamp = "<<fs.getFrameNumber()<<" "<<fs.getReferenceTime()<<std::endl;
         }
@@ -300,9 +323,13 @@ class DataConverter
             writeUInt(event.getEventType());
             writeUInt(event.getKey());
             writeUInt(event.getButton());
+            writeInt(event.getWindowX());
+            writeInt(event.getWindowY());
+            writeUInt(event.getWindowWidth());
+            writeUInt(event.getWindowHeight());
             writeFloat(event.getXmin());
-            writeFloat(event.getXmax());
             writeFloat(event.getYmin());
+            writeFloat(event.getXmax());
             writeFloat(event.getYmax());
             writeFloat(event.getX());
             writeFloat(event.getY());
@@ -316,10 +343,16 @@ class DataConverter
             event.setEventType((osgGA::GUIEventAdapter::EventType)readUInt());
             event.setKey(readUInt());
             event.setButton(readUInt());
-            event.setXmin(readFloat());
-            event.setXmax(readFloat());
-            event.setYmin(readFloat());
-            event.setYmax(readFloat());
+            int x = readInt();
+            int y = readInt();
+            int width = readUInt();
+            int height = readUInt();
+            event.setWindowRectangle(x,y,width,height);
+            float xmin = readFloat();
+            float ymin = readFloat();
+            float xmax = readFloat();
+            float ymax = readFloat();
+            event.setInputRange(xmin,ymin,xmax,ymax);
             event.setX(readFloat());
             event.setY(readFloat());
             event.setButtonMask(readUInt());
@@ -369,21 +402,20 @@ class DataConverter
         }
 };
 
-void CameraPacket::readEventQueue(osgProducer::Viewer& viewer)
+void CameraPacket::readEventQueue(osgViewer::Viewer& viewer)
 {
     _events.clear();
 
-    viewer.getKeyboardMouseCallback()->copyEventQueue(_events);
+    viewer.getEventQueue()->copyEvents(_events);
 
     osg::notify(osg::INFO)<<"written events = "<<_events.size()<<std::endl;
 }
 
-void CameraPacket::writeEventQueue(osgProducer::Viewer& viewer)
+void CameraPacket::writeEventQueue(osgViewer::Viewer& viewer)
 {
     osg::notify(osg::INFO)<<"recieved events = "<<_events.size()<<std::endl;
 
-    // copy the events to osgProducer style events.
-    viewer.getKeyboardMouseCallback()->appendEventQueue(_events);
+    viewer.getEventQueue()->appendEvents(_events);
 }
 
 
@@ -401,7 +433,7 @@ int main( int argc, char **argv )
     osg::ArgumentParser arguments(&argc,argv);
     
     // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates how to approach implementation of clustering. Note, cluster support will soon be encompassed in Producer itself.");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates how to approach implementation of clustering.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
     arguments.getApplicationUsage()->addCommandLineOption("-m","Set viewer to MASTER mode, sending view via packets.");
@@ -411,13 +443,7 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("-o <float>","Offset angle of camera");
     
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer;
 
 
     // read up the osgcluster specific arguments.
@@ -459,29 +485,35 @@ int main( int argc, char **argv )
         arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
         return 1;
     }
-    
-    
+
     // load model.
     osg::ref_ptr<osg::Node> rootnode = osgDB::readNodeFiles(arguments);
 
-    
     // set the scene to render
     viewer.setSceneData(rootnode.get());
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-
-    // set up the lens after realize as the Producer lens is not set up properly before this.... will need to inveestigate this at a later date.
     if (camera_fov>0.0f)
     {
-        float aspectRatio = tan( osg::DegreesToRadians(viewer.getLensVerticalFov()*0.5)) / tan(osg::DegreesToRadians(viewer.getLensHorizontalFov()*0.5));
-        float new_fovy = osg::RadiansToDegrees(atan( aspectRatio * tan( osg::DegreesToRadians(camera_fov*0.5))))*2.0f;
-        std::cout << "setting lens perspective : original "<<viewer.getLensHorizontalFov()<<"  "<<viewer.getLensVerticalFov()<<std::endl;
-        viewer.setLensPerspective(camera_fov,new_fovy,1.0f,1000.0f);
-        std::cout << "setting lens perspective : new "<<viewer.getLensHorizontalFov()<<"  "<<viewer.getLensVerticalFov()<<std::endl;
+        double fovy, aspectRatio, zNear, zFar;
+        viewer.getCamera()->getProjectionMatrixAsPerspective(fovy, aspectRatio,zNear, zFar);
+        
+        double original_fov = atan(tan(osg::DegreesToRadians(fovy)*0.5)*aspectRatio)*2.0;
+        std::cout << "setting lens perspective : original "<<original_fov<<"  "<<fovy<<std::endl;
+        
+        fovy = atan(tan(osg::DegreesToRadians(camera_fov)*0.5)/aspectRatio)*2.0;
+        viewer.getCamera()->setProjectionMatrixAsPerspective(fovy, aspectRatio,zNear, zFar);
+    
+        viewer.getCamera()->getProjectionMatrixAsPerspective(fovy, aspectRatio,zNear, zFar);
+        original_fov = atan(tan(osg::DegreesToRadians(fovy)*0.5)*aspectRatio)*2.0;
+        std::cout << "setting lens perspective : new "<<original_fov<<"  "<<fovy<<std::endl;
     }
 
+    viewer.setCameraManipulator(new osgGA::TrackballManipulator());
+
+
+    // create the windows and run the threads.
+    viewer.realize();
+
 
     CameraPacket *cp = new CameraPacket;
 
@@ -498,11 +530,10 @@ int main( int argc, char **argv )
 
     while( !viewer.done() && !masterKilled )
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
         osg::Timer_t startTick = osg::Timer::instance()->tick();
                  
+        viewer.advance();
+
         // special handling for working as a cluster.
         switch (viewerMode)
         {
@@ -510,7 +541,7 @@ int main( int argc, char **argv )
             {
                 
                 // take camera zero as the guide.
-                osg::Matrix modelview(viewer.getCamera(0)->getViewMatrix());
+                osg::Matrix modelview(viewer.getCamera()->getViewMatrix());
                 
                 cp->setPacket(modelview,viewer.getFrameStamp());
                 
@@ -561,31 +592,23 @@ int main( int argc, char **argv )
 
         // update the scene by traversing it with the the update visitor which will
         // call all node update callbacks and animations.
-        viewer.update();
+        viewer.eventTraversal();
+        viewer.updateTraversal();
 
         if (viewerMode==SLAVE)
         {
             osg::Matrix modelview;
             cp->getModelView(modelview,camera_offset);
         
-            viewer.setView(modelview);
+            viewer.getCamera()->setViewMatrix(modelview);
         }
 
         // fire off the cull and draw traversals of the scene.
         if(!masterKilled)
-            viewer.frame();
+            viewer.renderingTraversals();
         
     }
 
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
     // if we are master clean up by telling all slaves that we're going down.
     if (viewerMode==MASTER)
     {
