@@ -42,6 +42,7 @@
 #include <osgText/Text>
 
 #include <osgViewer/Viewer>
+#include <osgGA/StateSetManipulator>
 
 #include <iostream>
 
@@ -101,7 +102,7 @@ public:
         
         bool divide(unsigned int maxNumTreesPerCell=10);
         
-        bool devide(bool xAxis, bool yAxis, bool zAxis);
+        bool divide(bool xAxis, bool yAxis, bool zAxis);
         
         void bin();
 
@@ -243,7 +244,7 @@ bool ForestTechniqueManager::Cell::divide(unsigned int maxNumTreesPerCell)
 
     float radius = _bb.radius();
     float divide_distance = radius*0.7f;
-    if (devide((_bb.xMax()-_bb.xMin())>divide_distance,(_bb.yMax()-_bb.yMin())>divide_distance,(_bb.zMax()-_bb.zMin())>divide_distance))
+    if (divide((_bb.xMax()-_bb.xMin())>divide_distance,(_bb.yMax()-_bb.yMin())>divide_distance,(_bb.zMax()-_bb.zMin())>divide_distance))
     {
         // recusively divide the new cells till maxNumTreesPerCell is met.
         for(CellList::iterator citr=_cells.begin();
@@ -260,7 +261,7 @@ bool ForestTechniqueManager::Cell::divide(unsigned int maxNumTreesPerCell)
    }
 }
 
-bool ForestTechniqueManager::Cell::devide(bool xAxis, bool yAxis, bool zAxis)
+bool ForestTechniqueManager::Cell::divide(bool xAxis, bool yAxis, bool zAxis)
 {
     if (!(xAxis || yAxis || zAxis)) return false;
 
@@ -980,7 +981,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
     _techniqueSwitch = new osg::Switch;
 
     {
-        std::cout<<"Creating billboard based forest...";
+        std::cout<<"Creating osg::Billboard based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createBillboardGraph(cell.get(),dstate));
         group->addChild(createHUDWithText("Using osg::Billboard's to create a forest\n\nPress left cursor key to select OpenGL shader based forest\nPress right cursor key to select double quad based forest"));
@@ -989,7 +990,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
     }
     
     {
-        std::cout<<"Creating billboard based forest...";
+        std::cout<<"Creating double quad based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createXGraph(cell.get(),dstate));
         group->addChild(createHUDWithText("Using double quads to create a forest\n\nPress left cursor key to select osg::Billboard based forest\nPress right cursor key to select osg::MatrixTransform based forest\n"));
@@ -998,10 +999,10 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
     }
 
     {
-        std::cout<<"Creating billboard based forest...";
+        std::cout<<"Creating osg::MatrixTransform based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createTransformGraph(cell.get(),dstate));
-        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select OpenGL shder based forest"));
+        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select OpenGL shader based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
@@ -1065,7 +1066,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
             stateset->addUniform(baseTextureSampler);
         }
 
-        std::cout<<"Creating billboard based forest...";
+        std::cout<<"Creating OpenGL shader based forest...";
         group->addChild(createShaderGraph(cell.get(),stateset));
         group->addChild(createHUDWithText("Using OpenGL Shader to create a forest\n\nPress left cursor key to select osg::MatrixTransform based forest\nPress right cursor key to select osg::Billboard based forest"));
         _techniqueSwitch->addChild(group);
@@ -1091,7 +1092,7 @@ int main( int argc, char **argv )
     osg::ArgumentParser arguments(&argc,argv);
    
     // construct the viewer.
-    osgViewer::Viewer viewer;
+    osgViewer::Viewer viewer(arguments);
 
     float numTreesToCreates = 10000;
     arguments.read("--trees",numTreesToCreates);
@@ -1099,6 +1100,7 @@ int main( int argc, char **argv )
     osg::ref_ptr<ForestTechniqueManager> ttm = new ForestTechniqueManager;
     
     viewer.addEventHandler(new TechniqueEventHandler(ttm.get()));
+    viewer.addEventHandler(new osgGA::StateSetManipulator(viewer.getCamera()->getOrCreateStateSet()));
 
     // add model to viewer.
     viewer.setSceneData( ttm->createScene((unsigned int)numTreesToCreates) );
