@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgparametric.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Vec3>
 #include <osg/Vec4>
 #include <osg/Quat>
@@ -12,7 +30,10 @@
 
 #include <osgUtil/Optimizer>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
+#include <osgViewer/ViewerEventHandlers>
+
+#include <iostream>
 
 // for the grid data..
 #include "../osghangglide/terrain_coords.h"
@@ -80,12 +101,12 @@ class UniformVarying : public osg::Uniform::Callback
     virtual void operator () (osg::Uniform* uniform, osg::NodeVisitor* nv)
     {
         const osg::FrameStamp* fs = nv->getFrameStamp();
-        float value = sinf(fs->getReferenceTime());
+        float value = sinf(fs->getSimulationTime());
         uniform->set(osg::Vec4(value,-value,-value,value));
     }
 };
 
-osg::Node* createModel(const std::string& shader, const std::string& textureFileName, const std::string& terrainFileName)
+osg::Node* createModel(const std::string& shader, const std::string& textureFileName, const std::string& terrainFileName, bool dynamic, bool vbo)
 {
     osg::Geode* geode = new osg::Geode;
     
@@ -110,8 +131,16 @@ osg::Node* createModel(const std::string& shader, const std::string& textureFile
             program->addShader(vertex_shader);
 
             osg::Uniform* coeff = new osg::Uniform("coeff",osg::Vec4(1.0,-1.0f,-1.0f,1.0f));
-            coeff->setUpdateCallback(new UniformVarying);
+            
             stateset->addUniform(coeff);
+
+            if (dynamic)
+            {
+                coeff->setUpdateCallback(new UniformVarying);
+                coeff->setDataVariance(osg::Object::DYNAMIC);
+                stateset->setDataVariance(osg::Object::DYNAMIC);
+            }
+            
         }
         else if (shader=="matrix")
         {
@@ -209,6 +238,11 @@ osg::Node* createModel(const std::string& shader, const std::string& textureFile
 
     geom->setVertexArray(vertices);
 
+    osg::VertexBufferObject* vbObject = new osg::VertexBufferObject;
+    vertices->setVertexBufferObject(vbObject);
+    
+    osg::ElementBufferObject* ebo = new osg::ElementBufferObject;
+
     for(iy=0; iy<num_y-1; ++iy)
     {
         unsigned int element_no = 0;
@@ -219,10 +253,12 @@ osg::Node* createModel(const std::string& shader, const std::string& textureFile
             (*elements)[element_no++] = index + num_x;
             (*elements)[element_no++] = index++;
         }
-        geom->addPrimitiveSet(elements);    
+        geom->addPrimitiveSet(elements); 
+        
+        if (ebo) elements->setElementBufferObject(ebo);   
     }
     
-    // geom->setUseVertexBufferObjects(true);
+    geom->setUseVertexBufferObjects(vbo);
 
     return geode;
 }
@@ -238,11 +274,11 @@ int main(int argc, char *argv[])
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
    
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
+    osgViewer::Viewer viewer;
+
+    // add the stats handler
+    viewer.addEventHandler(new osgViewer::StatsHandler);
 
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-    
     std::string shader("simple");
     while(arguments.read("-s",shader)) {}
     
@@ -252,8 +288,11 @@ int main(int argc, char *argv[])
     std::string terrainFileName("");
     while(arguments.read("-d",terrainFileName)) {}
 
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    bool dynamic = true;
+    while(arguments.read("--static")) { dynamic = false; }
+
+    bool vbo = false;
+    while(arguments.read("--vbo")) { vbo = true; }
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -261,52 +300,15 @@ int main(int argc, char *argv[])
         arguments.getApplicationUsage()->write(std::cout);
         return 1;
     }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-
+    
     // load the nodes from the commandline arguments.
-    osg::Node* model = createModel(shader,textureFileName,terrainFileName);
+    osg::Node* model = createModel(shader,textureFileName,terrainFileName, dynamic, vbo);
     if (!model)
     {
         return 1;
     }
-
-    // add a viewport to the viewer and attach the scene graph.
+    
     viewer.setSceneData(model);
-    
-    // create the windows and run the threads.
-    viewer.realize();
 
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
