@@ -1,13 +1,19 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This application is open source and may be redistributed and/or modified under  
- * the terms of the GNU Public License (GPL) version 1.0 or 
- * (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * OpenSceneGraph Public License for more details.
+/* OpenSceneGraph example, osgphotoalbum.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
 #include "PhotoArchive.h"
@@ -15,32 +21,54 @@
 #include <osg/GLU>
 #include <osg/Notify>
 #include <osgDB/ReadFile>
-#include <Producer/RenderSurface>
+
+#include <osg/GraphicsContext>
 
 #include <fstream>
+#include <iostream>
 
 
 
 const std::string FILE_IDENTIFER("osgphotoalbum photo archive");
 
-class GraphicsContext {
+class MyGraphicsContext {
     public:
-        GraphicsContext()
+        MyGraphicsContext()
         {
-            rs = new Producer::RenderSurface;
-            rs->setWindowRectangle(0,0,1,1);
-            rs->useBorder(false);
-            rs->useConfigEventThread(false);
-            rs->realize();
-            std::cout<<"Realized window"<<std::endl;
-        }
+            osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
+            traits->x = 0;
+            traits->y = 0;
+            traits->width = 1;
+            traits->height = 1;
+            traits->windowDecoration = false;
+            traits->doubleBuffer = false;
+            traits->sharedContext = 0;
+            traits->pbuffer = true;
 
-        virtual ~GraphicsContext()
-        {
+            _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
+
+            if (!_gc)
+            {
+                osg::notify(osg::NOTICE)<<"Failed to create pbuffer, failing back to normal graphics window."<<std::endl;
+                
+                traits->pbuffer = false;
+                _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
+            }
+
+            if (_gc.valid()) 
+            
+            
+            {
+                _gc->realize();
+                _gc->makeCurrent();
+                std::cout<<"Realized window"<<std::endl;
+            }
         }
         
+        bool valid() const { return _gc.valid() && _gc->isRealized(); }
+        
     private:
-        Producer::ref_ptr<Producer::RenderSurface> rs;
+        osg::ref_ptr<osg::GraphicsContext> _gc;
 };
 
 PhotoArchive::PhotoArchive(const std::string& filename)
@@ -180,7 +208,7 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
     std::cout<<"Building photo archive containing "<<photoIndex.size()<<" pictures"<<std::endl;
 
     // create a graphics context so we can do data operations
-    GraphicsContext context;
+    MyGraphicsContext context;
 
     // open up the archive for writing to    
     std::ofstream out(filename.c_str(), std::ios::out | std::ios::binary);
