@@ -1,24 +1,33 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial applications,
- * as long as this copyright notice is maintained.
- * 
- * This application is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+/* OpenSceneGraph example, osgprecipitation.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
 #include <osgUtil/Optimizer>
 #include <osgUtil/CullVisitor>
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osg/MatrixTransform>
 #include <osgUtil/TransformCallback>
 #include <osgParticle/PrecipitationEffect>
 
+#include <iostream>
+
 class MyGustCallback : public osg::NodeCallback
 {
 
@@ -30,7 +39,7 @@ class MyGustCallback : public osg::NodeCallback
         {
             osgParticle::PrecipitationEffect* pe = dynamic_cast<osgParticle::PrecipitationEffect*>(node);
             
-            float value = sin(nv->getFrameStamp()->getReferenceTime());
+            float value = sin(nv->getFrameStamp()->getSimulationTime());
             if (value<-0.5)
             {
                 pe->snow(1.0);
@@ -56,26 +65,23 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" example provides an interactive viewer for visualising point clouds..");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
-    arguments.getApplicationUsage()->addCommandLineOption("","");
+    arguments.getApplicationUsage()->addCommandLineOption("--snow <density>","Set the snow with a density between 0 and 1.0");
+    arguments.getApplicationUsage()->addCommandLineOption("--rain <density>","");
+    arguments.getApplicationUsage()->addCommandLineOption("--particleSize <size>","");
+    arguments.getApplicationUsage()->addCommandLineOption("--particleColour <red> <green> <blue> <alpha>","");
+    arguments.getApplicationUsage()->addCommandLineOption("--wind <x> <y> <z>","Set the wind speed in model coordinates");
+    arguments.getApplicationUsage()->addCommandLineOption("--particleSpeed <float>","Set the particle speed");
+    arguments.getApplicationUsage()->addCommandLineOption("--nearTransition <distance>","Set the near transistion distance");
+    arguments.getApplicationUsage()->addCommandLineOption("--farTransition  <distance>","Set the far transistion distance");
+    arguments.getApplicationUsage()->addCommandLineOption("--particleDensity <density>","Set the particle density");
+    arguments.getApplicationUsage()->addCommandLineOption("--cellSize <x> <y> <z>","Set the cell size in model coordinates");
+    arguments.getApplicationUsage()->addCommandLineOption("--fogDensity <density>","Set the fog density");
+    arguments.getApplicationUsage()->addCommandLineOption("--fogColour <red> <green> <blue> <alpha>","Set fog colour.");
+    arguments.getApplicationUsage()->addCommandLineOption("-useFarLineSegments","Switch on the use of line segments");
     
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer;
 
     osg::ref_ptr<osgParticle::PrecipitationEffect> precipitationEffect = new osgParticle::PrecipitationEffect;
 
@@ -118,7 +124,7 @@ int main( int argc, char **argv )
     while (arguments.read("--useFarLineSegments")) { precipitationEffect->setUseFarLineSegments(true); }
 
     
-    viewer.setClearColor( precipitationEffect->getFog()->getColor() );
+    viewer.getCamera()->setClearColor( precipitationEffect->getFog()->getColor() );
 
 
     // if user request help write it out to cout.
@@ -127,16 +133,6 @@ int main( int argc, char **argv )
         arguments.getApplicationUsage()->write(std::cout);
         return 1;
     }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
     
     // read the scene from the list of file specified commandline args.
     osg::ref_ptr<osg::Node> loadedModel = osgDB::readNodeFiles(arguments);
@@ -170,32 +166,6 @@ int main( int argc, char **argv )
     // set the scene to render
     viewer.setSceneData(group.get());
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
 
