@@ -1,9 +1,27 @@
+/* OpenSceneGraph example, osgshape.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Geode>
 #include <osg/ShapeDrawable>
 #include <osg/Material>
 #include <osg/Texture2D>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osgDB/ReadFile>
 
@@ -12,7 +30,7 @@
 // for the grid data..
 #include "../osghangglide/terrain_coords.h"
 
-osg::Geode* createShapes( char* img_filename )
+osg::Geode* createShapes()
 {
     osg::Geode* geode = new osg::Geode();
 
@@ -21,8 +39,7 @@ osg::Geode* createShapes( char* img_filename )
     // ---------------------------------------
     osg::StateSet* stateset = new osg::StateSet();
 
-    if( ! img_filename ) img_filename = "Images/lz.rgb";
-    osg::Image* image = osgDB::readImageFile( img_filename );
+    osg::Image* image = osgDB::readImageFile( "Images/lz.rgb" );
 
     if (image)
     {
@@ -85,83 +102,13 @@ osg::Geode* createShapes( char* img_filename )
     return geode;
 }
 
-int main( int argc, char **argv )
+int main(int, char **)
 {
-
-    // use an ArgumentParser object to manage the program arguments.
-    osg::ArgumentParser arguments(&argc,argv);
-
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates the osg::Shape classes.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] [image_filename]");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-   
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-
-    char* img_filename = 0;
-    for( int pos = 1; pos < arguments.argc(); ++pos )
-    {
-        if( arguments.isString(pos) )
-        {
-            img_filename = arguments[pos];
-            break;
-        }
-    }
-    
-    osg::Node* node = createShapes( img_filename );
+    osgViewer::Viewer viewer;
 
     // add model to viewer.
-    viewer.setSceneData( node );
+    viewer.setSceneData( createShapes() );
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
