@@ -64,6 +64,8 @@
 #include <osgGA/AnimationPathManipulator>
 #include <osgGA/TerrainManipulator>
 
+#include <osgParticle/FireEffect>
+
 #include <iostream>
 
 osg::Node* createEarth()
@@ -290,9 +292,13 @@ int main(int argc, char **argv)
     while (arguments.read("--ortho") || arguments.read("--orthographic")) technique = osgSim::OverlayNode::VIEW_DEPENDENT_WITH_ORTHOGRAPHIC_OVERLAY;
     while (arguments.read("--persp") || arguments.read("--perspective")) technique = osgSim::OverlayNode::VIEW_DEPENDENT_WITH_PERSPECTIVE_OVERLAY;
 
+    unsigned int overlayTextureUnit = 1;
+    while (arguments.read("--unit", overlayTextureUnit)) {}
+    
     std::string pathfile;
     while (arguments.read("-p",pathfile)) {}
 
+    bool addFireEffect = arguments.read("--fire");
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -339,6 +345,7 @@ int main(int argc, char **argv)
             overlayNode->getOrCreateStateSet()->setTextureAttribute(1, new osg::TexEnv(osg::TexEnv::DECAL));
             overlayNode->setOverlaySubgraph(shapefile.get());
             overlayNode->setOverlayTextureSizeHint(1024);
+            overlayNode->setOverlayTextureUnit(overlayTextureUnit);
 
             // insert the OverlayNode between the coordinate system node and its children.
             for(unsigned int i=0; i<csn->getNumChildren(); ++i)
@@ -401,7 +408,16 @@ int main(int argc, char **argv)
                 osg::MatrixTransform* scaler = new osg::MatrixTransform;
                 scaler->addChild(cessna);
                 scaler->setMatrix(osg::Matrixd::scale(s,s,s)*osg::Matrixd::rotate(rotation));
-                scaler->getOrCreateStateSet()->setMode(GL_RESCALE_NORMAL,osg::StateAttribute::ON);        
+                scaler->getOrCreateStateSet()->setMode(GL_RESCALE_NORMAL,osg::StateAttribute::ON);
+                
+                if (addFireEffect)
+                {                
+                    osg::Vec3d center = cessna->getBound().center();
+                    
+                    osgParticle::FireEffect* fire = new osgParticle::FireEffect(center, 10.0f);
+                    scaler->addChild(fire);
+                }
+                
 
                 if (false)
                 {
