@@ -1,3 +1,19 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
+#include <stdlib.h>
+#include <string.h>
+
 #include <osg/ArgumentParser>
 #include <osg/ApplicationUsage>
 #include <osg/Notify>
@@ -19,6 +35,15 @@ bool ArgumentParser::isString(const char* str)
     //return !isOption(str);
 }
 
+bool ArgumentParser::isBool(const char* str)
+{
+    if (!str) return false;
+
+    return (strcmp(str,"True")==0 || strcmp(str,"true")==0 || strcmp(str,"TRUE")==0 ||
+            strcmp(str,"False")==0 || strcmp(str,"false")==0 || strcmp(str,"FALSE")==0 ||
+            strcmp(str,"0")==0 || strcmp(str,"1")==0);
+}
+
 bool ArgumentParser::isNumber(const char* str)
 {
     if (!str) return false;
@@ -117,6 +142,7 @@ bool ArgumentParser::Parameter::valid(const char* str) const
 {
     switch(_type)
     {
+    case Parameter::BOOL_PARAMETER:         return isBool(str); break;
     case Parameter::FLOAT_PARAMETER:        return isNumber(str); break;
     case Parameter::DOUBLE_PARAMETER:       return isNumber(str); break;
     case Parameter::INT_PARAMETER:          return isNumber(str); break;
@@ -132,6 +158,11 @@ bool ArgumentParser::Parameter::assign(const char* str)
     {
         switch(_type)
         {
+        case Parameter::BOOL_PARAMETER:
+        {
+            *_value._bool =  (strcmp(str,"True")==0 || strcmp(str,"true")==0 || strcmp(str,"TRUE")==0);
+            break;
+        }
         case Parameter::FLOAT_PARAMETER:        *_value._float = atof(str); break;
         case Parameter::DOUBLE_PARAMETER:       *_value._double = atof(str); break;
         case Parameter::INT_PARAMETER:          *_value._int = atoi(str); break;
@@ -552,7 +583,7 @@ void ArgumentParser::reportError(const std::string& message,ErrorSeverity severi
 void ArgumentParser::reportRemainingOptionsAsUnrecognized(ErrorSeverity severity)
 {
     std::set<std::string> options;
-    if (_usage) 
+    if (_usage.valid()) 
     {
         // parse the usage options to get all the option that the application can potential handle.
         for(ApplicationUsage::UsageMap::const_iterator itr=_usage->getCommandLineOptions().begin();
