@@ -28,7 +28,7 @@
 
 using namespace osg;
 
-// static cache of deleted display lists which can only 
+// static cache of deleted buffer object lists which can only 
 // by completely deleted once the appropriate OpenGL context
 // is set.  Used osg::BufferObject::deleteDisplayList(..) and flushDeletedBufferObjects(..) below.
 typedef std::multimap<unsigned int,GLuint> DisplayListMap;
@@ -48,8 +48,6 @@ void BufferObject::deleteBufferObject(unsigned int contextID,GLuint globj)
     }
 }
 
-/** flush all the cached display list which need to be deleted
-  * in the OpenGL context related to contextID.*/
 void BufferObject::flushDeletedBufferObjects(unsigned int contextID,double /*currentTime*/, double& availableTime)
 {
     // if no time available don't try to flush objects.
@@ -86,6 +84,13 @@ void BufferObject::flushDeletedBufferObjects(unsigned int contextID,double /*cur
     availableTime -= elapsedTime;
 }
 
+void BufferObject::discardDeletedBufferObjects(unsigned int contextID)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedBufferObjectCache);
+    DisplayListMap& dll = s_deletedBufferObjectCache[contextID];
+    dll.clear();
+}
+
 
 BufferObject::BufferObject():
     _target(0),
