@@ -29,6 +29,7 @@ CullSettings::CullSettings(const CullSettings& cs)
 void CullSettings::setDefaults()
 {
     _inheritanceMask = ALL_VARIABLES;
+    _inheritanceMaskActionOnAttributeSetting = DISABLE_ASSOCIATED_INHERITANCE_MASK_BIT;
     _cullingMode = DEFAULT_CULLING;
     _LODScale = 1.0f;
     _smallFeatureCullingPixelSize = 2.0f;
@@ -51,6 +52,7 @@ void CullSettings::setDefaults()
 void CullSettings::setCullSettings(const CullSettings& rhs)
 {
     _inheritanceMask = rhs._inheritanceMask;
+    _inheritanceMaskActionOnAttributeSetting = rhs._inheritanceMaskActionOnAttributeSetting;
 
     _computeNearFar = rhs._computeNearFar;
     _cullingMode = rhs._cullingMode;
@@ -144,3 +146,27 @@ void CullSettings::readCommandLine(ArgumentParser& arguments)
     }
 
 }
+
+void CullSettings::write(std::ostream& out)
+{
+    out<<"CullSettings: "<<this<<" {"<<std::endl;
+    
+    out<<"    _inheritanceMask = "<<_inheritanceMask<<std::endl;
+    out<<"    _inheritanceMaskActionOnAttributeSetting = "<<_inheritanceMaskActionOnAttributeSetting<<std::endl;
+    out<<"    _computeNearFar = "<<_computeNearFar<<std::endl;
+    out<<"    _cullingMode = "<<_cullingMode<<std::endl;
+    out<<"    _LODScale = "<<_LODScale<<std::endl;
+    out<<"    _smallFeatureCullingPixelSize = "<<_smallFeatureCullingPixelSize<<std::endl;
+    out<<"    _clampProjectionMatrixCallback = "<<_clampProjectionMatrixCallback.get()<<std::endl;
+    out<<"    _nearFarRatio = "<<_nearFarRatio<<std::endl;
+    out<<"    _impostorActive = "<<_impostorActive<<std::endl;
+    out<<"    _depthSortImpostorSprites = "<<_depthSortImpostorSprites<<std::endl;
+    out<<"    _impostorPixelErrorThreshold = "<<_impostorPixelErrorThreshold<<std::endl;
+    out<<"    _numFramesToKeepImpostorSprites = "<<_numFramesToKeepImpostorSprites<<std::endl;
+    out<<"    _cullMask = "<<_cullMask<<std::endl;
+    out<<"    _cullMaskLeft = "<<_cullMaskLeft<<std::endl;
+    out<<"    _cullMaskRight = "<<_cullMaskRight<<std::endl;
+    
+    out<<"{"<<std::endl;
+}
+
