@@ -132,6 +132,13 @@ void RenderBuffer::flushDeletedRenderBuffers(unsigned int contextID,double /*cur
     availableTime -= elapsedTime;
 }
 
+void RenderBuffer::discardDeletedRenderBuffers(unsigned int contextID)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedRenderBufferCache);
+    RenderBufferHandleList& pList = s_deletedRenderBufferCache[contextID];
+    pList.clear();
+}
+
 
 RenderBuffer::RenderBuffer()
 :    Object(),
@@ -191,7 +198,7 @@ GLuint RenderBuffer::getObjectID(unsigned int contextID, const FBOExtensions *ex
 }
 
 /**************************************************************************
- * FrameBufferAttachement
+ * FrameBufferAttachment
  **************************************************************************/
 
 #ifndef GL_TEXTURE_CUBE_MAP_POSITIVE_X
@@ -265,7 +272,7 @@ FrameBufferAttachment::FrameBufferAttachment(Texture2D* target, int level)
     _ximpl->textureTarget = target;
 }
 
-FrameBufferAttachment::FrameBufferAttachment(Texture3D* target, int level, int zoffset)
+FrameBufferAttachment::FrameBufferAttachment(Texture3D* target, int zoffset, int level)
 {
     _ximpl = new Pimpl(Pimpl::TEXTURE3D, level);
     _ximpl->textureTarget = target;
@@ -532,6 +539,14 @@ void FrameBufferObject::flushDeletedFrameBufferObjects(unsigned int contextID,do
     availableTime -= elapsedTime;
 }
 
+void FrameBufferObject::discardDeletedFrameBufferObjects(unsigned int contextID)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedFrameBufferObjectCache);
+    FrameBufferObjectHandleList& pList = s_deletedFrameBufferObjectCache[contextID];
+
+    pList.clear();
+}
+
 
 
 FrameBufferObject::FrameBufferObject()
@@ -541,7 +556,8 @@ FrameBufferObject::FrameBufferObject()
 
 FrameBufferObject::FrameBufferObject(const FrameBufferObject &copy, const CopyOp &copyop)
 :    StateAttribute(copy, copyop),
-    _attachments(copy._attachments)
+    _attachments(copy._attachments),
+    _drawBuffers(copy._drawBuffers)
 {
 }
 
@@ -553,6 +569,58 @@ FrameBufferObject::~FrameBufferObject()
     }
 }
 
+void FrameBufferObject::setAttachment(GLenum attachment_point, const FrameBufferAttachment &attachment)
+{
+    setAttachment(convertGLenumToBufferComponent(attachment_point),attachment);
+}
+
+void FrameBufferObject::setAttachment(BufferComponent attachment_point, const FrameBufferAttachment &attachment)
+{
+    GLenum gl_attachment = convertBufferComponentToGLenum(attachment_point);
+    _attachments[attachment_point] = attachment;
+
+    updateDrawBuffers();
+    dirtyAll();
+}
+
+
+GLenum FrameBufferObject::convertBufferComponentToGLenum(BufferComponent attachment_point) const
+{
+    switch(attachment_point)
+    {
+        case(Camera::DEPTH_BUFFER): return GL_DEPTH_ATTACHMENT_EXT;
+        case(Camera::STENCIL_BUFFER): return GL_STENCIL_ATTACHMENT_EXT;
+        case(Camera::COLOR_BUFFER): return GL_COLOR_ATTACHMENT0_EXT;
+        default: return GLenum(GL_COLOR_ATTACHMENT0_EXT + (attachment_point-Camera::COLOR_BUFFER0));
+    }
+}
+
+FrameBufferObject::BufferComponent FrameBufferObject::convertGLenumToBufferComponent(GLenum attachment_point) const
+{
+    switch(attachment_point)
+    {
+        case(GL_DEPTH_ATTACHMENT_EXT): return Camera::DEPTH_BUFFER;
+        case(GL_STENCIL_ATTACHMENT_EXT): return Camera::STENCIL_BUFFER;
+        case(GL_COLOR_ATTACHMENT0_EXT): return Camera::COLOR_BUFFER;
+        default: return BufferComponent(Camera::COLOR_BUFFER0+(attachment_point-GL_COLOR_ATTACHMENT0_EXT));
+    }
+}
+
+void FrameBufferObject::updateDrawBuffers()
+{
+    _drawBuffers.clear();
+
+    // create textures and mipmaps before we bind the frame buffer object
+    for (AttachmentMap::const_iterator i=_attachments.begin(); i!=_attachments.end(); ++i)
+    {
+        const FrameBufferAttachment &fa = i->second;
+
+        // setup draw buffers based on the attachment definition
+        if (i->first >= Camera::COLOR_BUFFER0 && i->first <= Camera::COLOR_BUFFER15)
+            _drawBuffers.push_back(convertBufferComponentToGLenum(i->first));
+    }
+}
+
 void FrameBufferObject::apply(State &state) const
 {
     unsigned int contextID = state.getContextID();
@@ -593,14 +661,13 @@ void FrameBufferObject::apply(State &state) const
 
     if (dirtyAttachmentList)
     {
-        // the set of of attachements appears to be thread sensitive, it shouldn't be because 
+        // the set of of attachments appears to be thread sensitive, it shouldn't be because 
         // OpenGL FBO handles osg::FrameBufferObject has are multi-buffered...
         // so as a temporary fix will stick in a mutex to ensure that only one thread passes through here
         // at one time.
         static OpenThreads::Mutex s_mutex;
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex);
 
-
         // create textures and mipmaps before we bind the frame buffer object
         for (AttachmentMap::const_iterator i=_attachments.begin(); i!=_attachments.end(); ++i)
         {
@@ -613,12 +680,22 @@ void FrameBufferObject::apply(State &state) const
    
     ext->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fboID);
 
+    // enable drawing buffers to render the result to fbo
+    if (_drawBuffers.size() > 0)
+    {
+        GL2Extensions *gl2e = GL2Extensions::Get(state.getContextID(), true );
+        if (gl2e)
+        {
+            gl2e->glDrawBuffers(_drawBuffers.size(), &(_drawBuffers[0]));
+        }
+    }
+
     if (dirtyAttachmentList)
     {
         for (AttachmentMap::const_iterator i=_attachments.begin(); i!=_attachments.end(); ++i)
         {
             const FrameBufferAttachment &fa = i->second;
-            fa.attach(state, i->first, ext);
+            fa.attach(state, convertBufferComponentToGLenum(i->first), ext);
         }        
         dirtyAttachmentList = 0;
     }
