@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -15,9 +15,9 @@
 
 using namespace osg;
 
-FrontFace::FrontFace()
+FrontFace::FrontFace(Mode face)
 {
-    _mode = COUNTER_CLOCKWISE;
+    _mode = face;
 }
 
 
