@@ -14,6 +14,7 @@
 #include <osg/Notify>
 #include <osg/GLU>
 #include <osg/GLExtensions>
+#include <osg/ApplicationUsage>
 
 #ifndef GL_MAX_TEXTURE_COORDS
 #define GL_MAX_TEXTURE_COORDS 0x8871
@@ -23,9 +24,15 @@
 #define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS 0x8B4D
 #endif
 
+#ifndef GL_MAX_TEXTURE_UNITS
+#define GL_MAX_TEXTURE_UNITS 0x84E2
+#endif
+
 using namespace std;
 using namespace osg;
 
+static ApplicationUsageProxy State_e0(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_GL_ERROR_CHECKING <type>","ONCE_PER_ATTRIBUTE | ON | on enables fine grained checking,  ONCE_PER_FRAME enables coarse grained checking");
+
 State::State():
     Referenced(true)
 {
@@ -38,11 +45,13 @@ State::State():
 
     _abortRenderingPtr = false;    
 
-#if 1
     _checkGLErrors = ONCE_PER_FRAME;
-#else    
-    _checkGLErrors = ONCE_PER_ATTRIBUTE;
-#endif
+
+    const char* str = getenv("OSG_GL_ERROR_CHECKING");
+    if (str && (strcmp(str,"ONCE_PER_ATTRIBUTE")==0 || strcmp(str,"ON")==0 || strcmp(str,"on")==0))
+    {
+        _checkGLErrors = ONCE_PER_ATTRIBUTE;
+    }
 
     _currentActiveTextureUnit=0;
     _currentClientActiveTextureUnit=0;
@@ -72,6 +81,9 @@ State::State():
     _glDisableVertexAttribArray = 0;
 
     _dynamicObjectCount  = 0;
+
+    _glMaxTextureCoords = 1;
+    _glMaxTextureUnits = 1;
 }
 
 State::~State()
@@ -110,7 +122,7 @@ void State::reset()
         as.changed = true;
     }
     
-    // we can do a straight clear, we arn't intrested in GL_DEPTH_TEST defaults in texture modes.
+    // we can do a straight clear, we arn't interested in GL_DEPTH_TEST defaults in texture modes.
     for(TextureModeMapList::iterator tmmItr=_textureModeMapList.begin();
         tmmItr!=_textureModeMapList.end();
         ++tmmItr)
@@ -147,7 +159,7 @@ void State::reset()
     // reset active texture unit values and call OpenGL
     // note, this OpenGL op precludes the use of State::reset() without a
     // valid graphics context, therefore the new implementation below 
-    // is prefered.
+    // is preferred.
     setActiveTextureUnit(0);
 #else
     // reset active texture unit values without calling OpenGL
@@ -365,7 +377,7 @@ void State::apply(const StateSet* dstate)
 {
     if (_checkGLErrors==ONCE_PER_ATTRIBUTE) checkGLErrors("start of State::apply(StateSet*)");
 
-    // equivilant to:
+    // equivalent to:
     //pushStateSet(dstate);
     //apply();
     //popStateSet();
@@ -428,7 +440,7 @@ void State::apply(const StateSet* dstate)
     }
     else
     {
-        // no incomming stateset, so simply apply state.
+        // no incoming stateset, so simply apply state.
         apply();
     }
 
@@ -742,12 +754,25 @@ void State::initializeExtensionProcs()
     _glDisableVertexAttribArray =  (DisableVertexAttribProc) osg::getGLExtensionFuncPtr("glDisableVertexAttribArray","glDisableVertexAttribArrayARB");
     _glBindBuffer = (BindBufferProc) osg::getGLExtensionFuncPtr("glBindBuffer","glBindBufferARB");
 
-    _glMaxTextureCoords = 1;
-    glGetIntegerv(GL_MAX_TEXTURE_COORDS,&_glMaxTextureCoords);
-
-    _glMaxTextureUnits = 1;
-    glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,&_glMaxTextureUnits);
-    _glMaxTextureUnits = maximum(_glMaxTextureCoords,_glMaxTextureUnits);
+    if ( osg::getGLVersionNumber() >= 2.0 || osg::isGLExtensionSupported(_contextID,"GL_ARB_vertex_shader") )
+    {
+        glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,&_glMaxTextureUnits);
+        glGetIntegerv(GL_MAX_TEXTURE_COORDS,&_glMaxTextureCoords);
+    }
+    else if ( osg::getGLVersionNumber() >= 1.3 ||
+                                 osg::isGLExtensionSupported(_contextID,"GL_ARB_multitexture") ||
+                                 osg::isGLExtensionSupported(_contextID,"GL_EXT_multitexture") )
+    {
+        GLint maxTextureUnits;
+        glGetIntegerv(GL_MAX_TEXTURE_UNITS,&maxTextureUnits);
+        _glMaxTextureUnits = maxTextureUnits;
+        _glMaxTextureCoords = maxTextureUnits;
+    }
+    else
+    {
+        _glMaxTextureUnits = 1;
+        _glMaxTextureCoords = 1;
+    }
 
     _extensionProcsInitialized = true;
 }
@@ -776,7 +801,7 @@ bool State::setActiveTextureUnit( unsigned int unit )
 {
     if (unit!=_currentActiveTextureUnit)
     {
-        if (_glActiveTexture && unit < (unsigned int)_glMaxTextureUnits)
+        if (_glActiveTexture && unit < (unsigned int)(maximum(_glMaxTextureCoords,_glMaxTextureUnits)) )
         {
             _glActiveTexture(GL_TEXTURE0+unit);
             _currentActiveTextureUnit = unit;
