@@ -141,10 +141,10 @@ StateSet::StateSet(const StateSet& rhs,const CopyOp& copyop):Object(rhs,copyop)
         rhs_uitr != rhs._uniformList.end();
         ++rhs_uitr)
     {
-    const std::string& name = rhs_uitr->first;
-    const RefUniformPair& rup = rhs_uitr->second;
-    Uniform* uni = copyop(rup.first.get());
-    if (uni)
+        const std::string& name = rhs_uitr->first;
+        const RefUniformPair& rup = rhs_uitr->second;
+        Uniform* uni = copyop(rup.first.get());
+        if (uni)
         {
             _uniformList[name] = RefUniformPair(uni, rup.second);
             uni->addParent(this);
@@ -607,9 +607,11 @@ void StateSet::merge(const StateSet& rhs)
         ModeList::iterator lhs_mitr = _modeList.find(rhs_mitr->first);
         if (lhs_mitr!=_modeList.end())
         {
-            if (!(lhs_mitr->second & StateAttribute::OVERRIDE)) 
+            // take the rhs mode unless the lhs is override and the rhs is not protected 
+            if (!(lhs_mitr->second & StateAttribute::OVERRIDE ) ||
+                 (rhs_mitr->second & StateAttribute::PROTECTED)) 
             {
-                // override isn't on in rhs, so overrite it with incomming
+                // override isn't on in rhs, so override it with incoming
                 // value.
                 lhs_mitr->second = rhs_mitr->second;
             }
@@ -630,16 +632,18 @@ void StateSet::merge(const StateSet& rhs)
         AttributeList::iterator lhs_aitr = _attributeList.find(rhs_aitr->first);
         if (lhs_aitr!=_attributeList.end())
         {
-            if (!(lhs_aitr->second.second & StateAttribute::OVERRIDE)) 
+            // take the rhs attribute unless the lhs is override and the rhs is not protected 
+            if (!(lhs_aitr->second.second & StateAttribute::OVERRIDE) ||
+                 (rhs_aitr->second.second & StateAttribute::PROTECTED))
             {
-                // override isn't on in rhs, so overrite it with incomming
+                // override isn't on in rhs, so override it with incoming
                 // value.
                 if (lhs_aitr->second.first!=rhs_aitr->second.first)
                 {
                     // new attribute so need to remove self from outgoing attribute
                     lhs_aitr->second.first->removeParent(this);
 
-                    // override isn't on in rhs, so overrite it with incomming
+                    // override isn't on in rhs, so override it with incoming
                     // value.
                     lhs_aitr->second = rhs_aitr->second;
                     lhs_aitr->second.first->addParent(this);
@@ -675,9 +679,11 @@ void StateSet::merge(const StateSet& rhs)
             ModeList::iterator lhs_mitr = lhs_modeList.find(rhs_mitr->first);
             if (lhs_mitr!=lhs_modeList.end())
             {
-                if (!(lhs_mitr->second & StateAttribute::OVERRIDE)) 
+                // take the rhs mode unless the lhs is override and the rhs is not protected 
+                if (!(lhs_mitr->second & StateAttribute::OVERRIDE) ||
+                     (rhs_mitr->second & StateAttribute::PROTECTED)) 
                 {
-                    // override isn't on in rhs, so overrite it with incomming
+                    // override isn't on in rhs, so override it with incoming
                     // value.
                     lhs_mitr->second = rhs_mitr->second;
                 }
@@ -705,9 +711,11 @@ void StateSet::merge(const StateSet& rhs)
             AttributeList::iterator lhs_aitr = lhs_attributeList.find(rhs_aitr->first);
             if (lhs_aitr!=lhs_attributeList.end())
             {
-                if (!(lhs_aitr->second.second & StateAttribute::OVERRIDE)) 
+                // take the rhs attribute unless the lhs is override and the rhs is not protected 
+                if (!(lhs_aitr->second.second & StateAttribute::OVERRIDE) ||
+                     (rhs_aitr->second.second & StateAttribute::PROTECTED)) 
                 {
-                    // override isn't on in rhs, so overrite it with incomming
+                    // override isn't on in rhs, so override it with incoming
                     // value.
                     
                     if (lhs_aitr->second.first!=rhs_aitr->second.first)
@@ -739,9 +747,11 @@ void StateSet::merge(const StateSet& rhs)
         UniformList::iterator lhs_uitr = _uniformList.find(rhs_uitr->first);
         if (lhs_uitr!=_uniformList.end())
         {
-            if (!(lhs_uitr->second.second & StateAttribute::OVERRIDE)) 
+            // take the rhs uniform unless the lhs is override and the rhs is not protected 
+            if (!(lhs_uitr->second.second & StateAttribute::OVERRIDE) ||
+                 (rhs_uitr->second.second & StateAttribute::PROTECTED)) 
             {
-                // override isn't on in rhs, so overrite it with incomming
+                // override isn't on in rhs, so override it with incoming
                 // value.
 
                 if (lhs_uitr->second.first!=rhs_uitr->second.first)
@@ -1461,18 +1471,55 @@ class SetAssociateModesHelper : public StateAttribute::ModeUsage
         unsigned int                _unit;
 };
 
+class RemoveAssociateModesHelper : public StateAttribute::ModeUsage
+{
+    public:
+        RemoveAssociateModesHelper(StateSet* stateset, unsigned int unit=0):
+            _stateset(stateset),
+            _unit(unit) {}
+            
+        virtual ~RemoveAssociateModesHelper() {}
+        
+        virtual void usesMode(StateAttribute::GLMode mode)
+        {
+            _stateset->removeMode(mode);
+        }
+        
+        virtual void usesTextureMode(StateAttribute::GLMode mode)
+        {
+           _stateset->removeTextureMode(_unit, mode);
+        }
+        
+       
+        
+        StateSet*                   _stateset;
+        unsigned int                _unit;
+};
+
 void StateSet::setAssociatedModes(const StateAttribute* attribute, StateAttribute::GLModeValue value)
 {
     SetAssociateModesHelper helper(this,value);
     attribute->getModeUsage(helper);
 }
 
+void StateSet::removeAssociatedModes(const StateAttribute* attribute)
+{
+    RemoveAssociateModesHelper helper(this);
+    attribute->getModeUsage(helper);
+}
+
 void StateSet::setAssociatedTextureModes(unsigned int unit,const StateAttribute* attribute, StateAttribute::GLModeValue value)
 {
     SetAssociateModesHelper helper(this,value,unit);
     attribute->getModeUsage(helper);
 }
 
+void StateSet::removeAssociatedTextureModes(unsigned int unit,const StateAttribute* attribute)
+{
+    RemoveAssociateModesHelper helper(this,unit);
+    attribute->getModeUsage(helper);
+}
+
 void StateSet::setAttribute(AttributeList& attributeList,StateAttribute *attribute, const StateAttribute::OverrideValue value)
 {
     if (attribute)
@@ -1743,7 +1790,7 @@ void StateSet::setNumChildrenRequiringUpdateTraversal(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -1788,7 +1835,7 @@ void StateSet::setNumChildrenRequiringEventTraversal(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
