@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -19,7 +19,7 @@ Stencil::Stencil()
     // set up same defaults as glStencilFunc.
     _func = ALWAYS;
     _funcRef = 0;
-    _funcMask = ~0;
+    _funcMask = ~0u;
         
     // set up same defaults as glStencilOp.
     _sfail = KEEP;
@@ -27,7 +27,7 @@ Stencil::Stencil()
     _zpass = KEEP;
 
     // set up same defaults as glStencilMask.
-    _writeMask = ~0;
+    _writeMask = ~0u;
 }
 
 
