@@ -108,7 +108,7 @@ namespace
 
                 tc->push_back(uv * scale);
             }
-            return tc.take();
+            return tc.release();
         }
 
     private:
@@ -428,7 +428,7 @@ namespace
         void getRequiredExtensions(std::vector<std::string>& extensions) const
         {
             extensions.push_back("GL_ARB_vertex_program");
-            extensions.push_back("GL_ARB__textureenv_dot3");
+            extensions.push_back("GL_ARB_texture_env_dot3");
         }
 
         void define_passes()
@@ -504,8 +504,8 @@ namespace
             }
 
             // second pass, self-shadowing
-	    bool selfShadowing = false; 
-	    if (selfShadowing)
+            bool selfShadowing = false; 
+            if (selfShadowing)
             {
                 std::ostringstream vp_oss;
                 vp_oss <<
