@@ -50,12 +50,14 @@ void TrackballManipulator::home(double /*currentTime*/)
 {
     if (getAutoComputeHomePosition()) computeHomePosition();
     computePosition(_homeEye, _homeCenter, _homeUp);
+    _thrown = false;
 }
 
 void TrackballManipulator::home(const GUIEventAdapter& ea ,GUIActionAdapter& us)
 {
     home(ea.getTime());
     us.requestRedraw();
+    us.requestContinuousUpdate(false);
 }
 
 
@@ -74,6 +76,20 @@ void TrackballManipulator::getUsage(osg::ApplicationUsage& usage) const
 
 bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 {
+    switch(ea.getEventType())
+    {
+        case(GUIEventAdapter::FRAME):
+            if (_thrown)
+            {
+                if (calcMovement()) us.requestRedraw();
+            }
+            return false;
+        default:
+            break;
+    }
+
+    if (ea.getHandled()) return false;
+
     switch(ea.getEventType())
     {
         case(GUIEventAdapter::PUSH):
@@ -90,6 +106,9 @@ bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us
         {
             if (ea.getButtonMask()==0)
             {
+            
+                double timeSinceLastRecordEvent = _ga_t0.valid() ? (ea.getTime() - _ga_t0->getTime()) : DBL_MAX;
+                if (timeSinceLastRecordEvent>0.02) flushMouseEventStack();
 
                 if (isMouseMoving())
                 {
@@ -141,8 +160,6 @@ bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us
                 flushMouseEventStack();
                 _thrown = false;
                 home(ea,us);
-                us.requestRedraw();
-                us.requestContinuousUpdate(false);
                 return true;
             }
             return false;
@@ -155,6 +172,7 @@ bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us
         default:
             return false;
     }
+    return false;
 }
 
 
@@ -233,9 +251,15 @@ bool TrackballManipulator::calcMovement()
     float dx = _ga_t0->getXnormalized()-_ga_t1->getXnormalized();
     float dy = _ga_t0->getYnormalized()-_ga_t1->getYnormalized();
 
-
+    float distance = sqrtf(dx*dx + dy*dy);
+    // return if movement is too fast, indicating an error in event values or change in screen.
+    if (distance>0.5)
+    {
+        return false;
+    }
+    
     // return if there is no movement.
-    if (dx==0.0f && dy==0.0f)
+    if (distance==0.0f)
     {
         return false;
     }
