@@ -168,24 +168,24 @@ PrecipitationEffect::PrecipitationEffect(const PrecipitationEffect& copy, const
     update();
 }
 
-void PrecipitationEffect::compileGLObjects(osg::State& state) const
+void PrecipitationEffect::compileGLObjects(osg::RenderInfo& renderInfo) const
 {
     if (_quadGeometry.valid()) 
     {
-        _quadGeometry->compileGLObjects(state);
-        if (_quadGeometry->getStateSet()) _quadGeometry->getStateSet()->compileGLObjects(state);
+        _quadGeometry->compileGLObjects(renderInfo);
+        if (_quadGeometry->getStateSet()) _quadGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
     }
 
     if (_lineGeometry.valid()) 
     {
-        _lineGeometry->compileGLObjects(state);
-        if (_lineGeometry->getStateSet()) _lineGeometry->getStateSet()->compileGLObjects(state);
+        _lineGeometry->compileGLObjects(renderInfo);
+        if (_lineGeometry->getStateSet()) _lineGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
     }
 
     if (_pointGeometry.valid()) 
     {
-        _pointGeometry->compileGLObjects(state);
-        if (_pointGeometry->getStateSet()) _pointGeometry->getStateSet()->compileGLObjects(state);
+        _pointGeometry->compileGLObjects(renderInfo);
+        if (_pointGeometry->getStateSet()) _pointGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
     }
 }
 
@@ -198,7 +198,7 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
     
         if (nv.getFrameStamp())
         {
-            double currentTime = nv.getFrameStamp()->getReferenceTime();
+            double currentTime = nv.getFrameStamp()->getSimulationTime();
             static double previousTime = currentTime;
             double delta = currentTime - previousTime;
             _origin += _wind * delta;
@@ -217,7 +217,7 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
         {
             if (globjVisitor->getMode() & osgUtil::GLObjectsVisitor::COMPILE_STATE_ATTRIBUTES)
             {
-                compileGLObjects(*(globjVisitor->getState()));
+                compileGLObjects(globjVisitor->getRenderInfo());
             }
         }
     
@@ -236,7 +236,7 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
         return;
     }
 
-    ViewIdenitifier viewIndentifier(cv, nv.getNodePath());
+    ViewIdentifier viewIndentifier(cv, nv.getNodePath());
     
     {
         PrecipitationDrawableSet* precipitationDrawableSet = 0;
@@ -275,21 +275,21 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
         if (!precipitationDrawableSet->_quadPrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_quadPrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_quadPrecipitationDrawable.get(),&cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_quadPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
             cv->popStateSet();
         }
 
         if (!precipitationDrawableSet->_linePrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_linePrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_linePrecipitationDrawable.get(),&cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_linePrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
             cv->popStateSet();
         }
 
         if (!precipitationDrawableSet->_pointPrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_pointPrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_pointPrecipitationDrawable.get(),&cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_pointPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
             cv->popStateSet();
         }
 
@@ -515,8 +515,8 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
             "\n"
-            "uniform float osg_FrameTime;\n"
-            "uniform float osg_DeltaFrameTime;\n"
+            "uniform float osg_SimulationTime;\n"
+            "uniform float osg_DeltaSimulationTime;\n"
             "\n"
             "varying vec4 colour;\n"
             "varying vec2 texCoord;\n"
@@ -528,10 +528,10 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    texCoord = gl_MultiTexCoord0.xy;\n"
             "\n"
             "    vec4 v_previous = gl_Vertex;\n"
-            "    v_previous.z = fract( (osg_FrameTime - startTime)*inversePeriod - offset);\n"
+            "    v_previous.z = fract( (osg_SimulationTime - startTime)*inversePeriod - offset);\n"
             "    \n"
             "    vec4 v_current =  v_previous;\n"
-            "    v_current.z += (osg_DeltaFrameTime*inversePeriod);\n"
+            "    v_current.z += (osg_DeltaSimulationTime*inversePeriod);\n"
             "    \n"
             "\n"
             "    colour = particleColour;\n"
@@ -589,8 +589,8 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
             "\n"
-            "uniform float osg_FrameTime;\n"
-            "uniform float osg_DeltaFrameTime;\n"
+            "uniform float osg_SimulationTime;\n"
+            "uniform float osg_DeltaSimulationTime;\n"
             "uniform mat4 previousModelViewMatrix;\n"
             "\n"
             "varying vec4 colour;\n"
@@ -603,10 +603,10 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    texCoord = gl_MultiTexCoord0.xy;\n"
             "\n"
             "    vec4 v_previous = gl_Vertex;\n"
-            "    v_previous.z = fract( (osg_FrameTime - startTime)*inversePeriod - offset);\n"
+            "    v_previous.z = fract( (osg_SimulationTime - startTime)*inversePeriod - offset);\n"
             "    \n"
             "    vec4 v_current =  v_previous;\n"
-            "    v_current.z += (osg_DeltaFrameTime*inversePeriod);\n"
+            "    v_current.z += (osg_DeltaSimulationTime*inversePeriod);\n"
             "    \n"
             "    colour = particleColour;\n"
             "    \n"
@@ -659,7 +659,7 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
             "\n"
-            "uniform float osg_FrameTime;\n"
+            "uniform float osg_SimulationTime;\n"
             "\n"
             "varying vec4 colour;\n"
             "\n"
@@ -669,7 +669,7 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    float startTime = gl_MultiTexCoord1.x;\n"
             "\n"
             "    vec4 v_current = gl_Vertex;\n"
-            "    v_current.z = fract( (osg_FrameTime - startTime)*inversePeriod - offset);\n"
+            "    v_current.z = fract( (osg_SimulationTime - startTime)*inversePeriod - offset);\n"
             "   \n"
             "    colour = particleColour;\n"
             "\n"
@@ -731,7 +731,7 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
     pds._pointPrecipitationDrawable->newFrame();
 
     osg::Matrix inverse_modelview;
-    inverse_modelview.invert(cv->getModelViewMatrix());
+    inverse_modelview.invert(*(cv->getModelViewMatrix()));
     
     osg::Vec3 eyeLocal = osg::Vec3(0.0f,0.0f,0.0f) * inverse_modelview;
     //osg::notify(osg::NOTICE)<<"  eyeLocal "<<eyeLocal<<std::endl;
@@ -746,8 +746,8 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
     
     osg::Polytope frustum;
     frustum.setToUnitFrustum(false,false);
-    frustum.transformProvidingInverse(cv->getProjectionMatrix());
-    frustum.transformProvidingInverse(cv->getModelViewMatrix());
+    frustum.transformProvidingInverse(*(cv->getProjectionMatrix()));
+    frustum.transformProvidingInverse(*(cv->getModelViewMatrix()));
 
     float i_delta = _farTransition * _inverse_du.x();
     float j_delta = _farTransition * _inverse_dv.y();
@@ -836,11 +836,11 @@ bool PrecipitationEffect::build(const osg::Vec3 eyeLocal, int i, int j, int k, f
         return false;
     }
 
-    *mymodelview = cv->getModelViewMatrix();
+    *mymodelview = *(cv->getModelViewMatrix());
     mymodelview->preMult(osg::Matrix::translate(position));
     mymodelview->preMult(osg::Matrix::scale(scale));
     
-    cv->updateCalculatedNearFar(cv->getModelViewMatrix(),bb);
+    cv->updateCalculatedNearFar(*(cv->getModelViewMatrix()),bb);
 
     return true;
 }
@@ -871,18 +871,18 @@ PrecipitationEffect::PrecipitationDrawable::PrecipitationDrawable(const Precipit
 
 
 
-void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::State& state) const
+void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderInfo& renderInfo) const
 {
     if (!_geometry) return;
 
-    const osg::Geometry::Extensions* extensions = osg::Geometry::getExtensions(state.getContextID(),true);
+    const osg::Geometry::Extensions* extensions = osg::Geometry::getExtensions(renderInfo.getContextID(),true);
     
     // save OpenGL matrices
     glPushMatrix();
     
     if (_requiresPreviousMatrix)
     {
-        state.setActiveTextureUnit(0);
+        renderInfo.getState()->setActiveTextureUnit(0);
         glMatrixMode( GL_TEXTURE );
         glPushMatrix();
     }
@@ -931,7 +931,7 @@ void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::State&
             glLoadMatrix((*itr)->second.modelview.ptr());
         }
 
-        _geometry->draw(state);
+        _geometry->draw(renderInfo);
         
         unsigned int numVertices = osg::minimum(_geometry->getVertexArray()->getNumElements(), _numberOfVertices);
         glDrawArrays(_drawType, 0, numVertices);
