@@ -11,18 +11,25 @@ class ReaderWriterIV : public osgDB::ReaderWriter
         
         virtual const char* className() const
         { 
-            return "Inventor Reader"; 
+            return "Inventor reader/writer"; 
         }
         
+        bool isInventorExtension(const std::string& extension) const
+        {
+            return osgDB::equalCaseInsensitive(extension, "iv") ? true : false;
+        }
+
         virtual bool acceptsExtension(const std::string& extension) const
         {
-            return osgDB::equalCaseInsensitive(extension, "iv") ? true :
+            return isInventorExtension(extension) ? true :
                    osgDB::equalCaseInsensitive(extension, "wrl") ? true : false;
         }
         
         virtual ReadResult readNode(const std::string& filename, 
                                     const osgDB::ReaderWriter::Options *) const;
 
+        virtual WriteResult writeNode(const osg::Node& node, const std::string& filename,
+                                      const osgDB::ReaderWriter::Options* options = NULL) const;
 };
 
 #endif
