@@ -1,19 +1,31 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 //
 // OpenFlight loader for OpenSceneGraph
 //
-//  Copyright (C) 2005-2006  Brede Johansen
+//  Copyright (C) 2005-2007  Brede Johansen
 //
 
 #ifndef FLT_DATAINPUTSTREAM
 #define FLT_DATAINPUTSTREAM 1
 
-//#include <iostream>        // for ifstream
 #include <istream>
 #include <string>
 #include <osg/Vec3f>
 #include <osg/Vec3d>
 #include <osg/Vec4f>
-#include "types.h"    
+#include "Types.h"    
 
 namespace flt {
 
@@ -47,9 +59,6 @@ class DataInputStream : public std::istream
 
     protected:
 
-        virtual std::istream& vread(char_type *str, std::streamsize count);
-        virtual std::istream& vforward(std::istream::off_type off);
-
         bool                _byteswap;
 };
 
