@@ -1,7 +1,20 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 //
 // OpenFlight loader for OpenSceneGraph
 //
-//  Copyright (C) 2005-2006  Brede Johansen
+//  Copyright (C) 2005-2007  Brede Johansen
 //
 
 #include <assert.h>
@@ -17,6 +30,7 @@
 #include <osgSim/DOFTransform>
 #include <osgSim/MultiSwitch>
 #include <osgSim/GeographicLocation>
+#include <osgSim/ObjectRecordData>
 #include "Registry.h"
 #include "Document.h"
 #include "RecordInputStream.h"
@@ -41,7 +55,6 @@ public:
 
     META_setID(_header)
     META_setComment(_header)
-//  META_setMatrix(_header)
     META_setMultitexture(_header)
     META_addChild(_header)
 
@@ -119,7 +132,7 @@ protected:
         document.setHeaderNode(_header.get());
     }
 
-    virtual void popLevel(Document& document)
+    virtual void dispose(Document& document)
     {
         if (_header.valid())
         {
@@ -173,7 +186,6 @@ public:
 
     META_setID(_group)
     META_setComment(_group)
-    META_setMatrix(_group)
     META_setMultitexture(_group)
     META_addChild(_group)
 
@@ -223,8 +235,16 @@ protected:
             _parent->addChild(*_group);
     }
 
-    virtual void popLevel(Document& document)
+    virtual void dispose(Document& document)
     {
+        if (!_group.valid()) return;
+
+        // Insert transform(s)
+        if (_matrix.valid())
+        {
+            insertMatrixTransform(*_group,*_matrix,_numberOfReplications);
+        }
+
         // Children are added!
         osg::Sequence* sequence = dynamic_cast<osg::Sequence*>(_group.get());
         if (sequence && sequence->getNumChildren() > 0)
@@ -305,14 +325,13 @@ public:
 
     META_setID(_dof)
     META_setComment(_dof)
-    META_setMatrix(_dof)
     META_setMultitexture(_dof)
     META_addChild(_dof)
+    META_dispose(_dof)
 
 protected:
 
     virtual ~DegreeOfFreedom() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
@@ -440,14 +459,13 @@ public:
 
     META_setID(_lod)
     META_setComment(_lod)
-    META_setMatrix(_lod)
     META_setMultitexture(_lod)
     META_addChild(_impChild0)
+    META_dispose(_lod)
 
 protected:
 
     virtual ~LevelOfDetail() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
@@ -496,14 +514,13 @@ public:
 
     META_setID(_lod)
     META_setComment(_lod)
-    META_setMatrix(_lod)
     META_setMultitexture(_lod)
     META_addChild(_impChild0)
+    META_dispose(_lod)
 
 protected:
 
     virtual ~OldLevelOfDetail() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
@@ -557,8 +574,8 @@ public:
 
     META_setID(_multiSwitch)
     META_setComment(_multiSwitch)
-    META_setMatrix(_multiSwitch)
     META_setMultitexture(_multiSwitch)
+    META_dispose(_multiSwitch)
 
     virtual void addChild(osg::Node& child)
     {
@@ -580,7 +597,6 @@ public:
 protected:
 
     virtual ~Switch() {}
-
     virtual void readRecord(RecordInputStream& in, Document& /*document*/)
     {
         std::string id = in.readString(8);
@@ -639,14 +655,13 @@ public:
 
     META_setID(_external)
     META_setComment(_external)
-    META_setMatrix(_external)
     META_setMultitexture(_external)
     META_addChild(_external)
+    META_dispose(_external)
 
 protected:
 
     virtual ~ExternalReference() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string strFile = in.readString(200);
@@ -704,11 +719,12 @@ RegisterRecordProxy<ExternalReference> g_ExternalReference(EXTERNAL_REFERENCE_OP
 */
 class InstanceDefinition : public PrimaryRecord
 {
+    int _number;
     osg::ref_ptr<osg::Group> _instanceDefinition;
 
 public:
 
-    InstanceDefinition() {}
+    InstanceDefinition():_number(0) {}
 
     META_Record(InstanceDefinition)
 
@@ -717,28 +733,32 @@ public:
     META_setMultitexture(_instanceDefinition)
     META_addChild(_instanceDefinition)
 
-    virtual void setMatrix(osg::Matrix& matrix)
-    {
-        osg::ref_ptr<osg::MatrixTransform> transform = new osg::MatrixTransform(matrix);
-        transform->setDataVariance(osg::Object::STATIC);
-        transform->addChild(_instanceDefinition.get());
-        _instanceDefinition = transform.get();
-    }
-
 protected:
 
     virtual ~InstanceDefinition() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         in.forward(2);
-        uint16 number = in.readUInt16();
+        _number = (int)in.readUInt16();
 
         _instanceDefinition = new osg::Group;
+    }
+
+    virtual void dispose(Document& document)
+    {
+        // Insert transform(s)
+        if (_matrix.valid())
+        {
+            osg::ref_ptr<osg::MatrixTransform> transform = new osg::MatrixTransform(*_matrix);
+            transform->setDataVariance(osg::Object::STATIC);
+            transform->addChild(_instanceDefinition.get());
+            _instanceDefinition = transform.get();
+        }
 
         //  Add to instance definition table.
-        document.setInstanceDefinition(number,_instanceDefinition.get());
+        document.setInstanceDefinition(_number,_instanceDefinition.get());
     }
+
 };
 
 RegisterRecordProxy<InstanceDefinition> g_InstanceDefinition(INSTANCE_DEFINITION_OP);
@@ -758,7 +778,6 @@ public:
 protected:
 
     virtual ~InstanceReference() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         in.forward(2);
@@ -768,7 +787,7 @@ protected:
         osg::Node* instance = document.getInstanceDefinition(number);
 
         // Add this implementation to parent implementation.
-        if (_parent.valid())
+        if (_parent.valid() && instance)
             _parent->addChild(*instance);
     }
 };
@@ -790,14 +809,13 @@ public:
 
     META_setID(_extension)
     META_setComment(_extension)
-    META_setMatrix(_extension)
     META_setMultitexture(_extension)
     META_addChild(_extension)
+    META_dispose(_extension)
 
 protected:
 
     virtual ~Extension() {}
-
     virtual void readRecord(RecordInputStream& in, Document& /*document*/)
     {
         std::string id = in.readString(8);
@@ -837,85 +855,84 @@ public:
 
     META_setID(_object)
     META_setComment(_object)
-
-    virtual void setMatrix(osg::Matrix& matrix)
-    {
-        if (_object.valid())
-            insertMatrixTransform(*_object,matrix);
-        else
-        {
-            _object = new osg::MatrixTransform(matrix);
-            _object->setDataVariance(osg::Object::STATIC);
-
-            if (_parent.valid())
-                _parent->addChild(*_object);
-        }
-    }
-
-    virtual void addChild(osg::Node& child)
-    {
-        // If object excists it means it is preserved.
-        if (_object.valid())
-            _object->addChild(&child);
-        // If no object add child to parent.
-        else if (_parent.valid())
-            _parent->addChild(child);
-    }
+    META_addChild(_object)
 
 protected:
 
-    virtual ~Object() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
-        // Is it safe to remove the object?
-        if (!document.getPreserveObject())
-        {
-            // The following tests need a valid parent.
-            if (!_parent.valid())
-                return;
+        std::string id = in.readString(8);
 
+        _object = new osg::Group;
+        _object->setName(id);
+
+        if (document.getReadObjectRecordData())
+        {
+            osgSim::ObjectRecordData* ord = new osgSim::ObjectRecordData;
+            ord->_flags = in.readUInt32();
+            ord->_relativePriority = in.readInt16();
+            ord->_transparency = in.readUInt16();
+            ord->_effectID1 = in.readInt16();
+            ord->_effectID2 = in.readInt16();
+            ord->_significance = in.readInt16();
+
+            _object->setUserData( ord );
+        }
+        else
+        {
+            /*uint32 flags =*/ in.readUInt32();
+        }
+        
+        // Postpone add-to-parent until we know a bit more.
+    }
+
+    virtual void dispose(Document& document)
+    {
+        if (!_parent.valid() || !_object.valid()) return;
+
+        // Is it safe to remove _object?
+        if (!document.getPreserveObject() && isSafeToRemoveObject() && !_matrix.valid())
+        {
+            // Add children of _object to parent.
+            // _object will not be added to graph.
+            for (unsigned int i=0; i<_object->getNumChildren(); ++i)
+            {
+                _parent->addChild(*(_object->getChild(i)));
+            }
+        }
+        else
+        {
+            _parent->addChild(*_object);
+        }
+
+        // Insert transform(s)
+        if (_matrix.valid())
+        {
+            insertMatrixTransform(*_object,*_matrix,_numberOfReplications);
+        }
+    }
+
+    bool isSafeToRemoveObject() const
+    {
+        // The following tests need a valid parent.
+        if (_parent.valid())
+        {
             // LODs adds an empty child group so it is safe to remove this object record.
             if (typeid(*_parent)==typeid(flt::LevelOfDetail))
-                return;
+                return true;
 
             if (typeid(*_parent)==typeid(flt::OldLevelOfDetail))
-                return;
+                return true;
 
             // If parent is a Group record we have to check for animation.
             Group* parentGroup = dynamic_cast<flt::Group*>(_parent.get());
             if (parentGroup && !parentGroup->hasAnimation())
-                return;
+                return true;
         }
 
-        std::string id = in.readString(8);
-        _object = new osg::Group;
-        _object->setName(id);
-
-
-#if 1
-        /*uint32 flags =*/ in.readUInt32();
-#else
-
-        // The Object "Flat Shaded" checkbox in Creator is used by the "Calculate Shading" operation,
-        // it is not a flat shaded state attribute.
-        
-        uint32 flags = in.readUInt32();
-        // Flat shaded?
-        if (flags & FLAT_SHADED)
-        {
-            static osg::ref_ptr<osg::ShadeModel> shademodel;
-            if (!shademodel.valid())
-            {
-                shademodel = new osg::ShadeModel;
-                shademodel->setMode(osg::ShadeModel::FLAT);
-            }
-            _object->getOrCreateStateSet()->setAttribute(shademodel.get());
-        }
-#endif
-        if (_parent.valid())
-            _parent->addChild(*_object);
+        return false;
     }
+
 };
 
 RegisterRecordProxy<Object> g_Object(OBJECT_OP);
@@ -939,12 +956,11 @@ public:
 
     META_setID(_lightSource)
     META_setComment(_lightSource)
-    META_setMatrix(_lightSource)
+    META_dispose(_lightSource)
 
 protected:
 
     virtual ~LightSource() {}
-
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
