@@ -1,7 +1,20 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 //
 // OpenFlight loader for OpenSceneGraph
 //
-//  Copyright (C) 2005-2006  Brede Johansen
+//  Copyright (C) 2005-2007  Brede Johansen
 //
 
 #include "Record.h"
@@ -228,7 +241,7 @@ class AbsoluteVertex : public Record
             vertex.setCoord(osg::Vec3(x,y,z) * document.unitScale());
 
             // optional texture coordinates
-            if (in.tellg() < in.getEndOfRecord())
+            if (in.getRecordBodySize() > (4+4+4))
             {
                 osg::Vec2f uv = in.readVec2f();
                 vertex.setUV(0,uv);
@@ -276,7 +289,7 @@ class ShadedVertex : public Record
                 vertex.setColor(getColorFromPool(colorIndex, document.getColorPool()));   // Color from pool
 
             // optional texture coordinates
-            if (in.tellg() < in.getEndOfRecord())
+            if (in.getRecordBodySize() > (4+4+4+1+1+2))
             {
                 osg::Vec2f uv = in.readVec2f();
                 vertex.setUV(0,uv);
@@ -324,7 +337,7 @@ class NormalVertex : public Record
                 vertex.setColor(getColorFromPool(colorIndex, document.getColorPool()));   // Color from pool
 
             // optional texture coordinates
-            if (in.tellg() < in.getEndOfRecord())
+            if (in.getRecordBodySize() > (4+4+4+1+1+2+3*8))
             {
                 osg::Vec2f uv = in.readVec2f();
                 vertex.setUV(0,uv);
