@@ -11,6 +11,7 @@
  * OpenSceneGraph Public License for more details.
 */
 
+#include <iostream>
 #include <fstream>
 #include <stdio.h>
 
@@ -112,6 +113,16 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
 }
 
 
+std::string Model::lastComponent(const char* linep)
+{
+    std::string line = std::string(linep);
+    int space = line.find_last_of(" ");
+    if (space >= 0) {
+        line = line.substr(space+1);
+    }
+    return line;
+}
+
 bool Model::readMTL(std::istream& fin)
 {
     osg::notify(osg::INFO)<<"Reading MTL file"<<std::endl;
@@ -213,13 +224,6 @@ bool Model::readMTL(std::istream& fin)
 
                     if (fieldsRead==1) material->Ni = Ni;
                 }
-                else if (strncmp(line,"illum ",6)==0)
-                {
-                    int illum = 0;
-                    unsigned int fieldsRead = sscanf(line+6,"%d", &illum);
-
-                    if (fieldsRead==1) material->illum = illum;
-                }
                 else if (strncmp(line,"Tr ",3)==0)
                 {
                     float alpha=1.0f;
@@ -253,13 +257,11 @@ bool Model::readMTL(std::istream& fin)
                 }
                 else if (strncmp(line,"map_Kd ",7)==0)
                 {
-                    std::string filename(line+7);
-                    material->map_Kd = filename;
+                    material->map_Kd = lastComponent(line+7);
                 }
                 else if (strncmp(line,"map_Ks ",7)==0)
                 {
-                    std::string filename(line+7);
-                    material->map_Ks = filename;
+                    material->map_Ks = lastComponent(line+7);
                 }
                 else if (strcmp(line,"refl")==0 || strncmp(line,"refl ",5)==0)
                 {
@@ -282,6 +284,17 @@ bool Model::readMTL(std::istream& fin)
     return true;
 }
 
+std::string trim(const std::string& s)
+{
+  if(s.length() == 0)
+    return s;
+  int b = s.find_first_not_of(" \t");
+  int e = s.find_last_not_of(" \t");
+  if(b == -1) // No non-spaces
+    return "";
+  return std::string(s, b, e - b + 1);
+}
+
 bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* options)
 {
     osg::notify(osg::INFO)<<"Reading OBJ file"<<std::endl;
@@ -402,7 +415,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
             }
             else if (strncmp(line,"usemtl ",7)==0)
             {
-                std::string materialName(line+7);
+                std::string materialName( line+7 );
                 if (currentElementState.materialName != materialName)
                 {
                     currentElementState.materialName = materialName;
@@ -411,11 +424,14 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
             }
             else if (strncmp(line,"mtllib ",7)==0)
             {
-            
-                std::string fileName = osgDB::findDataFile( line+7, options );
-                if (!fileName.empty())
+                std::string materialFileName = trim( line+7 );
+                std::string fullPathFileName = osgDB::findDataFile( materialFileName, options );
+                osg::notify(osg::INFO) << "--" << line+7 << "--" << std::endl;
+                osg::notify(osg::INFO) << "--" << materialFileName << "--" << std::endl;
+                osg::notify(osg::INFO) << "--" << fullPathFileName << "--" << std::endl;
+                if (!fullPathFileName.empty())
                 {
-                    std::ifstream mfin(fileName.c_str());
+                    std::ifstream mfin( fullPathFileName.c_str() );
                     if (mfin)
                     {
                         readMTL(mfin);
