@@ -78,6 +78,8 @@ void initGLNames()
     ADD_NAME("GL_POLYGON_OFFSET_LINE",GL_POLYGON_OFFSET_LINE)
     ADD_NAME("GL_POLYGON_OFFSET_POINT",GL_POLYGON_OFFSET_POINT)
     ADD_NAME("GL_COLOR_SUM",GL_COLOR_SUM);
+    ADD_NAME("GL_NORMALIZE",GL_NORMALIZE);
+    ADD_NAME("GL_RESCALE_NORMAL",GL_RESCALE_NORMAL);
     
     ADD_NAME("GL_TEXTURE_1D",GL_TEXTURE_1D)
     ADD_NAME("GL_TEXTURE_2D",GL_TEXTURE_2D)
