@@ -18,20 +18,20 @@
 #include <string.h>
 
 /* trpage_print.cpp
-    Print out the contents of a TerraPage archive.
-    This module provides an example of how to access each of the classes
-    within a TerraPage archive.
- */
+   Print out the contents of a TerraPage archive.
+   This module provides an example of how to access each of the classes
+   within a TerraPage archive.
+*/
 
 #include <trpage_print.h>
 
 /* ******************************************
    Print Buffer implementation
-    The print buffer is a way to dump debugging data out
-    to a file (or console).  You can make your own subclass
-    of trpgPrintBuffer if you have specific needs.
+   The print buffer is a way to dump debugging data out
+   to a file (or console).  You can make your own subclass
+   of trpgPrintBuffer if you have specific needs.
    ******************************************
- */
+   */
 
 trpgPrintBuffer::trpgPrintBuffer()
 {
@@ -96,7 +96,8 @@ bool trpgFilePrintBuffer::prnLine(char *str)
     if (!fp)
         return false;
 
-    if (str) {
+    if (str)
+    {
         fprintf(fp,indentStr);
         fprintf(fp,str);
         fprintf(fp,"\n");
@@ -112,7 +113,7 @@ bool trpgFilePrintBuffer::prnLine(char *str)
    is readable/writeable are here.  These are used for
    debugging.
    *************************************
- */
+   */
 
 /* Print out the header information.
  */
@@ -122,14 +123,20 @@ bool trpgHeader::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Archive Header----");
     buf.IncreaseIndent();
     sprintf(ls,"verMinor = %d, verMajor = %d",verMinor,verMajor); buf.prnLine(ls);
+    if((verMajor >= TRPG_NOMERGE_VERSION_MAJOR) && (verMinor >=TRPG_NOMERGE_VERSION_MINOR))
+    {
+        sprintf(ls,"isMaster = %s, numRows = %d, numCols = %d",GetIsMaster()?"YES":"NO",rows,cols); buf.prnLine(ls);
+    }
     sprintf(ls,"dbVerMinor = %d, dbVerMajor = %d",dbVerMinor,dbVerMajor); buf.prnLine(ls);
+
     sprintf(ls,"maxGroupID = %d",maxGroupID); buf.prnLine(ls);
     sprintf(ls,"sw = (%f,%f), ne = (%f,%f)",sw.x,sw.y,ne.x,ne.y); buf.prnLine(ls);
     sprintf(ls,"tileType = %d, origin = (%f,%f,%f)",tileType,origin.x,origin.y,origin.z); buf.prnLine(ls);
 
     sprintf(ls,"numLods = %d",numLods); buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (int i=0;i<numLods;i++) {
+    for (int i=0;i<numLods;i++)
+    {
         sprintf(ls,"tileSize = (%f,%f), lodSizes = (%d,%d), lodRanges = %f",tileSize[i].x,tileSize[i].y,lodSizes[i].x,lodSizes[i].y,lodRanges[i]); buf.prnLine(ls);
     }
     buf.DecreaseIndent(2);
@@ -176,7 +183,8 @@ bool trpgMaterial::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"numTile = %d",numTile); buf.prnLine(ls);
     sprintf(ls,"numTex = %d",numTex); buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (int i=0;i<numTex;i++) {
+    for (int i=0;i<numTex;i++)
+    {
         sprintf(ls,"texID[%d] = %d",i,texids[i]); buf.prnLine(ls);
         buf.IncreaseIndent();
         texEnvs[i].Print(buf);
@@ -199,16 +207,18 @@ bool trpgMatTable::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"numTable = %d",numTable); buf.prnLine(ls);
     sprintf(ls,"numMat = %d",numMat);  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (int j=0;j<numTable;j++) {
-        sprintf(ls,"----Sub Table %d----",j); buf.prnLine(ls);
-        buf.IncreaseIndent();
-        for (int i=0;i<numMat;i++) {
-            const trpgMaterial *mat;
-            sprintf(ls,"Material %d",i);  buf.prnLine(ls);
-            mat = (const_cast<trpgMatTable *>(this))->GetMaterialRef(j,i);
-            mat->Print(buf);
+    MaterialMapType::const_iterator itr = materialMap.begin();
+    for (  ; itr != materialMap.end( ); itr++)
+    {
+        const trpgMaterial *mat;
+        sprintf(ls,"Material %d",itr->first);  buf.prnLine(ls);
+        mat = (const_cast<trpgMatTable *>(this))->GetMaterialRef(0,itr->first);
+        if(!mat)
+        {
+            sprintf(ls,"Error: Unable to load material!");  buf.prnLine(ls);
         }
-        buf.DecreaseIndent();
+        else
+            mat->Print(buf);
     }
     buf.DecreaseIndent(2);
 
@@ -227,9 +237,10 @@ bool trpgTexture::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"Name = %s",name); buf.prnLine(ls);
     sprintf(ls,"useCount = %d",useCount);  buf.prnLine(ls);
     sprintf(ls,"sizeX = %d, sizeY = %d, sizeZ = %d",sizeX,sizeY,numLayer);  buf.prnLine(ls);
-    sprintf(ls,"sensor band organization = %d",org); buf.prnLine(ls);
+//    sprintf(ls,"sensor band organization = %d",org); buf.prnLine(ls); // does this need to be added?
     sprintf(ls,"ismipmap = %d",isMipmap);  buf.prnLine(ls);
     sprintf(ls,"addr.file = %d, addr.offset = %d",addr.file,addr.offset);  buf.prnLine(ls);
+    sprintf(ls,"addr.col = %d, addr.row = %d",addr.col,addr.row);  buf.prnLine(ls);
     buf.DecreaseIndent();
 
     buf.prnLine();
@@ -245,9 +256,11 @@ bool trpgTexTable::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Texture Table----");
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<texList.size();i++) {
-        sprintf(ls,"Texture %d",i); buf.prnLine(ls);
-        texList[i].Print(buf);
+    TextureMapType::const_iterator itr = textureMap.begin();
+    for (  ; itr != textureMap.end( ); itr++)
+    {
+        sprintf(ls,"Texture %d",itr->first); buf.prnLine(ls);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
 
@@ -264,9 +277,11 @@ bool trpgModelTable::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Model Table----");
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<models.size();i++) {
-        sprintf(ls,"Model %d",i);  buf.prnLine(ls);
-        models[i].Print(buf);
+    ModelMapType::const_iterator itr = modelsMap.begin();
+    for (  ; itr != modelsMap.end( ); itr++)
+    {
+        sprintf(ls,"Model %d",itr->first);  buf.prnLine(ls);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
 
@@ -284,7 +299,8 @@ bool trpgModel::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Model----");
     buf.IncreaseIndent();
     sprintf(ls,"type = %d",type); buf.prnLine(ls);
-    if (name) {
+    if (name)
+    {
         sprintf(ls,"name = %s",name); buf.prnLine(ls);
     }
     sprintf(ls,"diskRef = %d",(int)diskRef), buf.prnLine(ls);
@@ -305,22 +321,24 @@ bool trpgTileHeader::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Tile Header----");
     buf.IncreaseIndent();
 
-    sprintf(ls,"matList size = %d",matList.size()); buf.prnLine(ls);
+    sprintf(ls,"matList size = %d",static_cast<int>(matList.size())); buf.prnLine(ls);
     buf.IncreaseIndent();
     unsigned int i;
-    for (i=0;i<matList.size();i++) {
+    for (i=0;i<matList.size();i++)
+    {
         sprintf(ls,"matList[%d] = %d",i,matList[i]); buf.prnLine(ls);
     }
     buf.DecreaseIndent();
 
-    sprintf(ls,"modelList size = %d",modelList.size());  buf.prnLine(ls);
+    sprintf(ls,"modelList size = %d",static_cast<int>(modelList.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (i=0;i<modelList.size();i++) {
+    for (i=0;i<modelList.size();i++)
+    {
         sprintf(ls,"modelList[%d] = %d",i,modelList[i]); buf.prnLine(ls);
     }
     buf.DecreaseIndent();
 
-    sprintf(ls,"local material list size = %d",locMats.size());  buf.prnLine(ls);
+    sprintf(ls,"local material list size = %d",static_cast<int>(locMats.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
     for (i=0;i<locMats.size();i++)
         locMats[i].Print(buf);
@@ -342,9 +360,10 @@ bool trpgColorInfo::Print(trpgPrintBuffer &buf) const
     buf.IncreaseIndent();
 
     sprintf(ls,"type = %d, bind = %d",type,bind);  buf.prnLine(ls);
-    sprintf(ls,"colorData size = %d",data.size());
+    sprintf(ls,"colorData size = %d",static_cast<int>(data.size()));
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<data.size();i++) {
+    for (unsigned int i=0;i<data.size();i++)
+    {
         sprintf(ls,"color[%d] = (%f,%f,%f)",i,data[i].red,data[i].blue,data[i].green); buf.prnLine(ls);
     }
     buf.DecreaseIndent(2);
@@ -363,18 +382,24 @@ bool trpgTexData::Print(trpgPrintBuffer &buf) const
     buf.IncreaseIndent();
 
     sprintf(ls,"bind = %d",bind);  buf.prnLine(ls);
-    if (floatData.size()) {
-        sprintf(ls,"tex coords (float) = %d",floatData.size()); buf.prnLine(ls);
+    if (floatData.size())
+    {
+        sprintf(ls,"tex coords (float) = %d",static_cast<int>(floatData.size())); buf.prnLine(ls);
         buf.IncreaseIndent();
-        for (unsigned int i=0;i<floatData.size()/2;i++) {
+        for (unsigned int i=0;i<floatData.size()/2;i++)
+        {
             sprintf(ls,"tex coord[%d] = (%f,%f)",i,floatData[i*2+0],floatData[i*2+1]);  buf.prnLine(ls);
         }
         buf.DecreaseIndent();
-    } else {
-        if (doubleData.size()) {
-            sprintf(ls,"tex coords (double) = %d",doubleData.size());
+    }
+    else
+    {
+        if (doubleData.size())
+        {
+            sprintf(ls,"tex coords (double) = %d",static_cast<int>(doubleData.size()));
             buf.IncreaseIndent();
-            for (unsigned int i=0;i<doubleData.size()/2;i++) {
+            for (unsigned int i=0;i<doubleData.size()/2;i++)
+            {
                 sprintf(ls,"tex coord[%d] = (%f,%f)",i,doubleData[i*2+0],doubleData[i*2+1]),  buf.prnLine(ls);
             }
             buf.DecreaseIndent();
@@ -396,11 +421,12 @@ bool trpgGeometry::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Geometry Node----");
     buf.IncreaseIndent();
 
-    sprintf(ls,"Material size = %d",materials.size());  buf.prnLine(ls);
+    sprintf(ls,"Material size = %d",static_cast<int>(materials.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
     ls[0] = 0;
     unsigned int i;
-    for (i=0;i<materials.size();i++) {
+    for (i=0;i<materials.size();i++)
+    {
         char locStr[100];
         sprintf(locStr,"%d ",materials[i]);
         strcat(ls,locStr);
@@ -409,10 +435,11 @@ bool trpgGeometry::Print(trpgPrintBuffer &buf) const
     buf.DecreaseIndent();
 
     sprintf(ls,"primType = %d, numPrim = %d",primType,numPrim);  buf.prnLine(ls);
-    sprintf(ls,"primLength size = %d",primLength.size());  buf.prnLine(ls);
+    sprintf(ls,"primLength size = %d",static_cast<int>(primLength.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
     ls[0] = 0;
-    for (i=0;i<primLength.size();i++) {
+    for (i=0;i<primLength.size();i++)
+    {
         char locStr[100];
         sprintf(locStr,"%d ",primLength[i]);
         strcat(ls,locStr);
@@ -420,21 +447,27 @@ bool trpgGeometry::Print(trpgPrintBuffer &buf) const
     buf.prnLine(ls);
     buf.DecreaseIndent();
 
-    if (vertDataFloat.size()) {
-        sprintf(ls,"vert data (float) length = %d",vertDataFloat.size());
+    if (vertDataFloat.size())
+    {
+        sprintf(ls,"vert data (float) length = %d",static_cast<int>(vertDataFloat.size()));
         buf.prnLine(ls);
         buf.IncreaseIndent();
-        for (i=0;i<vertDataFloat.size()/3;i++) {
+        for (i=0;i<vertDataFloat.size()/3;i++)
+        {
             sprintf(ls,"(%f, %f, %f)",vertDataFloat[3*i],vertDataFloat[3*i+1],vertDataFloat[3*i+2]);
             buf.prnLine(ls);
         }
         buf.DecreaseIndent();
-    } else {
-        if (vertDataDouble.size()) {
-            sprintf(ls,"vert data (double) length = %d",vertDataDouble.size());
+    }
+    else
+    {
+        if (vertDataDouble.size())
+        {
+            sprintf(ls,"vert data (double) length = %d",static_cast<int>(vertDataDouble.size()));
             buf.prnLine(ls);
             buf.IncreaseIndent();
-            for (i=0;i<vertDataDouble.size()/3;i++) {
+            for (i=0;i<vertDataDouble.size()/3;i++)
+            {
                 sprintf(ls,"(%f, %f, %f)",vertDataDouble[3*i],vertDataDouble[3*i+1],vertDataDouble[3*i+2]);
                 buf.prnLine(ls);
             }
@@ -444,21 +477,27 @@ bool trpgGeometry::Print(trpgPrintBuffer &buf) const
 
     sprintf(ls,"normBind = %d",normBind);  buf.prnLine(ls);
 
-    if (normDataFloat.size()) {
-        sprintf(ls,"norm data (float) length = %d",normDataFloat.size());
+    if (normDataFloat.size())
+    {
+        sprintf(ls,"norm data (float) length = %d",static_cast<int>(normDataFloat.size()));
         buf.prnLine(ls);
         buf.IncreaseIndent();
-        for (i=0;i<normDataFloat.size()/3;i++) {
+        for (i=0;i<normDataFloat.size()/3;i++)
+        {
             sprintf(ls,"(%f, %f, %f)",normDataFloat[3*i],normDataFloat[3*i+1],normDataFloat[3*i+2]);
             buf.prnLine(ls);
         }
         buf.DecreaseIndent();
-    } else {
-        if (normDataDouble.size()) {
-            sprintf(ls,"norm data (double) length = %d",normDataDouble.size());
+    }
+    else
+    {
+        if (normDataDouble.size())
+        {
+            sprintf(ls,"norm data (double) length = %d",static_cast<int>(normDataDouble.size()));
             buf.prnLine(ls);
             buf.IncreaseIndent();
-            for (i=0;i<normDataDouble.size()/3;i++) {
+            for (i=0;i<normDataDouble.size()/3;i++)
+            {
                 sprintf(ls,"(%f, %f, %f)",normDataDouble[3*i],normDataDouble[3*i+1],normDataDouble[3*i+2]);
                 buf.prnLine(ls);
             }
@@ -466,16 +505,18 @@ bool trpgGeometry::Print(trpgPrintBuffer &buf) const
         }
     }
 
-    sprintf(ls,"color info size = %d",colors.size()); buf.prnLine(ls);
+    sprintf(ls,"color info size = %d",static_cast<int>(colors.size())); buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (i=0;i<colors.size();i++) {
+    for (i=0;i<colors.size();i++)
+    {
         colors[i].Print(buf);
     }
     buf.DecreaseIndent();
 
-    sprintf(ls,"tex data size = %d",texData.size());
+    sprintf(ls,"tex data size = %d",static_cast<int>(texData.size()));
     buf.IncreaseIndent();
-    for (i=0;i<texData.size();i++) {
+    for (i=0;i<texData.size();i++)
+    {
         texData[i].Print(buf);
     }
     buf.DecreaseIndent();
@@ -497,7 +538,7 @@ bool trpgGroup::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Group Node----");
     buf.IncreaseIndent();
     sprintf(ls,"id = %d, numChild = %d",id,numChild);  buf.prnLine(ls);
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
 
 
     buf.DecreaseIndent();
@@ -515,7 +556,25 @@ bool trpgAttach::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Attach Node----");
     buf.IncreaseIndent();
     sprintf(ls,"id = %d, parentID = %d, childPos = %d",id,parentID,childPos);  buf.prnLine(ls);
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
+
+    buf.DecreaseIndent();
+    buf.prnLine();
+
+    return true;
+}
+
+/* Print ChildRef info
+ */
+bool trpgChildRef::Print(trpgPrintBuffer &buf) const
+{
+    char ls[1024];
+
+    buf.prnLine("----RefChild Node----");
+    buf.IncreaseIndent();
+    sprintf(ls,"lod = %d, x = %d, y = %d", lod, x, y);	buf.prnLine(ls);
+    sprintf(ls,"file = %d, offset = %d", addr.file, addr.offset);	buf.prnLine(ls);
+    sprintf(ls,"zmin = %f, zmax = %f", zmin, zmax);	buf.prnLine(ls);
 
     buf.DecreaseIndent();
     buf.prnLine();
@@ -534,7 +593,7 @@ bool trpgBillboard::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"id = %d,  type = %d, mode = %d",id,type,mode);  buf.prnLine(ls);
     sprintf(ls,"center = (%f,%f,%f)",center.x,center.y,center.z);  buf.prnLine(ls);
     sprintf(ls,"axis = (%f,%f,%f)",axis.x,axis.y,axis.z);  buf.prnLine(ls);
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
 
     buf.DecreaseIndent();
     buf.prnLine();
@@ -554,7 +613,7 @@ bool trpgLod::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"numRange (hint) = %d",numRange);  buf.prnLine(ls);
     sprintf(ls,"switchIn = %f, switchOut = %f, width = %f",switchIn,switchOut,width);  buf.prnLine(ls);
     sprintf(ls,"center = (%f,%f,%f)",center.x,center.y,center.z);
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
     sprintf(ls,"rangeIndex = %d",rangeIndex);  buf.prnLine(ls);
 
     buf.DecreaseIndent();
@@ -572,7 +631,7 @@ bool trpgLayer::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Layer Node----");
     buf.IncreaseIndent();
     sprintf(ls,"id = %d",id);  buf.prnLine(ls);
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
 
     buf.DecreaseIndent();
     buf.prnLine();
@@ -590,12 +649,13 @@ bool trpgTransform::Print(trpgPrintBuffer &buf) const
     buf.IncreaseIndent();
     sprintf(ls,"id = %d",id);  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (int i=0;i<4;i++) {
+    for (int i=0;i<4;i++)
+    {
         sprintf(ls,"%f %f %f %f",m[i][0],m[i][1],m[i][2],m[i][3]);
         buf.prnLine(ls);
     }
-    sprintf(ls,"name = %s", name ? name : "noname" );    buf.prnLine(ls);
-    
+    sprintf(ls,"name = %s", name ? name : "noname" );	buf.prnLine(ls);
+	
     buf.DecreaseIndent(2);
     buf.prnLine();
 
@@ -612,11 +672,12 @@ bool trpgModelRef::Print(trpgPrintBuffer &buf) const
     buf.IncreaseIndent();
     sprintf(ls,"modelRef = %d",modelRef);  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (int i=0;i<4;i++) {
+    for (int i=0;i<4;i++)
+    {
         sprintf(ls,"%f %f %f %f",m[i][0],m[i][1],m[i][2],m[i][3]);
         buf.prnLine(ls);
     }
-    
+	
     buf.DecreaseIndent(2);
     buf.prnLine();
 
@@ -631,14 +692,30 @@ bool trpgTileTable::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Tile Table----");
     buf.IncreaseIndent();
-    sprintf(ls,"mode = %d",mode);  buf.prnLine(ls);
-    sprintf(ls,"numLod = %d",lodInfo.size());  buf.prnLine(ls);
-    for (unsigned int i=0;i<lodInfo.size();i++) {
+    switch(mode)
+    {
+    case Local:
+        sprintf(ls,"mode = %d(Local)",mode);  buf.prnLine(ls);
+        break;
+    case External:
+        sprintf(ls,"mode = %d(External)",mode);  buf.prnLine(ls);
+        break;
+    case ExternalSaved:
+        sprintf(ls,"mode = %d(ExternalSaved)",mode);  buf.prnLine(ls);
+        break;
+    default:
+        sprintf(ls,"mode = %d",mode);  buf.prnLine(ls);
+    }
+
+    sprintf(ls,"numLod = %d",static_cast<int>(lodInfo.size()));  buf.prnLine(ls);
+    for (unsigned int i=0;i<lodInfo.size();i++)
+    {
         const LodInfo &li = lodInfo[i];
         sprintf(ls,"LOD %d, numX = %d, numY = %d",i,li.numX,li.numY);  buf.prnLine(ls);
         buf.prnLine("File ID, Offset, Zmin, Zmax");
         buf.IncreaseIndent();
-        for (unsigned int j=0;j<li.addr.size();j++) {
+        for (unsigned int j=0;j<li.addr.size();j++)
+        {
             sprintf(ls,"%d %d %f %f",li.addr[j].file,li.addr[j].offset,li.elev_min[j],li.elev_max[j]);  buf.prnLine(ls);
         }
         buf.DecreaseIndent();
@@ -660,7 +737,8 @@ bool trpgLocalMaterial::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"baseMat = %d",baseMat);  buf.prnLine(ls);
     sprintf(ls,"(sx,sy) -> (ex,ey) = (%d,%d) -> (%d,%d)",sx,sy,ex,ey); buf.prnLine(ls);
     sprintf(ls,"dest (width,height) = (%d,%d)",destWidth,destHeight); buf.prnLine(ls);
-    for (unsigned int i=0;i<addr.size();i++) {
+    for (unsigned int i=0;i<addr.size();i++)
+    {
         sprintf(ls,"addr (file,offset) = (%d,%d)",addr[i].file,addr[i].offset); buf.prnLine(ls);
     }
     buf.DecreaseIndent();
@@ -670,7 +748,7 @@ bool trpgLocalMaterial::Print(trpgPrintBuffer &buf) const
 }
 
 /* Light Attribute Print
-*/
+ */
 bool trpgLightAttr::Print(trpgPrintBuffer &buf) const
 {
     char ls[1024];
@@ -681,84 +759,85 @@ bool trpgLightAttr::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Light Attribute----");
     buf.IncreaseIndent();
-    sprintf(ls,"type = %s",strType[(int)(data.type)]);                        buf.prnLine(ls);
-    sprintf(ls,"directionality = %s",strDirect[(int)(data.directionality)]);buf.prnLine(ls);
+    sprintf(ls,"type = %s",strType[(int)(data.type)]);				buf.prnLine(ls);
+    sprintf(ls,"directionality = %s",strDirect[(int)(data.directionality)]);    buf.prnLine(ls);
     sprintf(ls,"front color (RGB) = %.2lf, %.2lf, %.2lf",
-        data.frontColor.red, data.frontColor.green,data.frontColor.blue );    buf.prnLine(ls);
-    sprintf(ls,"front intensity = %.2lf", data.frontIntensity );            buf.prnLine(ls);
+            data.frontColor.red, data.frontColor.green,data.frontColor.blue );	buf.prnLine(ls);
+    sprintf(ls,"front intensity = %.2lf", data.frontIntensity );		buf.prnLine(ls);
     sprintf(ls,"back color (RGB) = %.2lf, %.2lf, %.2lf",
-        data.backColor.red, data.backColor.green,data.backColor.blue );        buf.prnLine(ls);
-    sprintf(ls,"back intensity = %.2lf", data.backIntensity );                buf.prnLine(ls);
+            data.backColor.red, data.backColor.green,data.backColor.blue );	buf.prnLine(ls);
+    sprintf(ls,"back intensity = %.2lf", data.backIntensity );			buf.prnLine(ls);
     sprintf(ls,"normal (xyz) = %.2lf,%.2lf,%.2lf",
-        data.normal.x,data.normal.y,data.normal.z );                        buf.prnLine(ls);
-    sprintf(ls,"smc = %d",data.smc);                                        buf.prnLine(ls);
-    sprintf(ls,"fid = %d",data.fid);                                        buf.prnLine(ls);
+            data.normal.x,data.normal.y,data.normal.z );			buf.prnLine(ls);
+    sprintf(ls,"smc = %d",data.smc);						buf.prnLine(ls);
+    sprintf(ls,"fid = %d",data.fid);						buf.prnLine(ls);
     sprintf(ls,"visible at DAY = %s",
-        (data.flags & trpgLightAttr::trpg_Day ? "yes" : "no") );                buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Day ? "yes" : "no") );		buf.prnLine(ls);
     sprintf(ls,"visible at DUSK = %s",
-        (data.flags & trpgLightAttr::trpg_Dusk ? "yes" : "no") );            buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Dusk ? "yes" : "no") );		buf.prnLine(ls);
     sprintf(ls,"visible at NIGHT = %s",
-        (data.flags & trpgLightAttr::trpg_Night ? "yes" : "no") );            buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Night ? "yes" : "no") );		buf.prnLine(ls);
     sprintf(ls,"enable directionality = %s",
-        (data.flags & trpgLightAttr::trpg_Directional ? "yes" : "no" ));        buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Directional ? "yes" : "no" ));	buf.prnLine(ls);
     sprintf(ls,"enable back color = %s",
-        (data.flags & trpgLightAttr::trpg_BackColor ? "yes" : "no" ));        buf.prnLine(ls);
-    sprintf(ls,"horizontal lobe angle = %.2lf",data.horizontalLobeAngle);    buf.prnLine(ls);
-    sprintf(ls,"vertical lobe angle = %.2lf",data.verticalLobeAngle);        buf.prnLine(ls);
-    sprintf(ls,"lobe roll angle = %.2lf",data.lobeRollAngle);                buf.prnLine(ls);
-    sprintf(ls,"lobe falloff = %.2lf",data.lobeFalloff);                    buf.prnLine(ls);
-    sprintf(ls,"ambient intensity = %.2lf",data.ambientIntensity);            buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_BackColor ? "yes" : "no" ));	buf.prnLine(ls);
+    sprintf(ls,"horizontal lobe angle = %.2lf",data.horizontalLobeAngle);	buf.prnLine(ls);
+    sprintf(ls,"vertical lobe angle = %.2lf",data.verticalLobeAngle);		buf.prnLine(ls);
+    sprintf(ls,"lobe roll angle = %.2lf",data.lobeRollAngle);			buf.prnLine(ls);
+    sprintf(ls,"lobe falloff = %.2lf",data.lobeFalloff);			buf.prnLine(ls);
+    sprintf(ls,"ambient intensity = %.2lf",data.ambientIntensity);		buf.prnLine(ls);
     sprintf(ls,"reflective only = %s",
-        (data.flags & trpgLightAttr::trpg_Reflective ? "yes" : "no") );        buf.prnLine(ls);
-    sprintf(ls,"quality = %s", strQuality[(int)(data.quality)]);            buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Reflective ? "yes" : "no") );	buf.prnLine(ls);
+    sprintf(ls,"quality = %s", strQuality[(int)(data.quality)]);		buf.prnLine(ls);
     sprintf(ls,"significance for RASCAL lights = %.2lf",
-        data.rascalSignificance );                                            buf.prnLine(ls);
+            data.rascalSignificance );						buf.prnLine(ls);
     sprintf(ls,"calligraphic draw order = %d",
-        data.calligraphicAttr.drawOrder );                                    buf.prnLine(ls);
+            data.calligraphicAttr.drawOrder );					buf.prnLine(ls);
     sprintf(ls,"calligraphic lights maximum defocus = %f",
-        data.calligraphicAttr.maxDefocus );                                    buf.prnLine(ls);
+            data.calligraphicAttr.maxDefocus );					buf.prnLine(ls);
     sprintf(ls,"calligraphic lights minimum defocus = %f",
-        data.calligraphicAttr.minDefocus );                                    buf.prnLine(ls);
+            data.calligraphicAttr.minDefocus );					buf.prnLine(ls);
     sprintf(ls,"randomize intensity = %s",
-        strQuality[(int)(data.randomIntensity)]);                            buf.prnLine(ls);
+            strQuality[(int)(data.randomIntensity)]);				buf.prnLine(ls);
     sprintf(ls,"performer perspective mode = %s",
-        (data.flags & trpgLightAttr::trpg_Perspective ? "yes" : "no" ) );    buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Perspective ? "yes" : "no" ) );	buf.prnLine(ls);
     sprintf(ls,"performer fade = %s",
-        (data.flags & trpgLightAttr::trpg_Fade ? "yes" : "no" ) );            buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Fade ? "yes" : "no" ) );		buf.prnLine(ls);
     sprintf(ls,"performer fog punch = %s",
-        (data.flags & trpgLightAttr::trpg_FogPunch ? "yes" : "no" ) );        buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_FogPunch ? "yes" : "no" ) );	buf.prnLine(ls);
     sprintf(ls,"performer range mode enable Z buffer = %s",
-        (data.flags & trpgLightAttr::trpg_ZBuffer ? "yes" : "no" ) );        buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_ZBuffer ? "yes" : "no" ) );	buf.prnLine(ls);
     sprintf(ls,"performer maximum pixel size = %.2lf",
-        data.performerAttr.maxPixelSize );                                    buf.prnLine(ls);
+            data.performerAttr.maxPixelSize );					buf.prnLine(ls);
     sprintf(ls,"performer minimum pixel size = %.2lf",
-        data.performerAttr.minPixelSize );                                    buf.prnLine(ls);
+            data.performerAttr.minPixelSize );					buf.prnLine(ls);
     sprintf(ls,"performer actual size = %.2lf",
-        data.performerAttr.actualSize );                                    buf.prnLine(ls);
+            data.performerAttr.actualSize );					buf.prnLine(ls);
     sprintf(ls,"performer transparent pixel size = %.2lf",
-        data.performerAttr.transparentPixelSize );                            buf.prnLine(ls);
+            data.performerAttr.transparentPixelSize );				buf.prnLine(ls);
     sprintf(ls,"performer transparent falloff exponent = %.2lf",
-        data.performerAttr.transparentFallofExp );                            buf.prnLine(ls);
+            data.performerAttr.transparentFallofExp );				buf.prnLine(ls);
     sprintf(ls,"performer transparent scale = %.2lf",
-        data.performerAttr.transparentScale );                                buf.prnLine(ls);
+            data.performerAttr.transparentScale );				buf.prnLine(ls);
     sprintf(ls,"performer transparent clamp = %.2lf",
-        data.performerAttr.transparentClamp );                                buf.prnLine(ls);
+            data.performerAttr.transparentClamp );				buf.prnLine(ls);
     sprintf(ls,"performer fog scale = %.2lf",
-        data.performerAttr.fogScale );                                        buf.prnLine(ls);
-    sprintf(ls,"animation period = %.2lf",data.animationAttr.period);        buf.prnLine(ls);
+            data.performerAttr.fogScale );					buf.prnLine(ls);
+    sprintf(ls,"animation period = %.2lf",data.animationAttr.period);		buf.prnLine(ls);
     sprintf(ls,"animation phase delay = %.2lf",
-        data.animationAttr.phaseDelay );                                    buf.prnLine(ls);
-    sprintf(ls,"animation time on = %.2lf",data.animationAttr.timeOn);        buf.prnLine(ls);
+            data.animationAttr.phaseDelay );					buf.prnLine(ls);
+    sprintf(ls,"animation time on = %.2lf",data.animationAttr.timeOn);		buf.prnLine(ls);
     sprintf(ls,"animation vector (ijk) = %.2lf, %.2lf, %.2lf",
-        data.animationAttr.vector.x,data.animationAttr.vector.y,
-        data.animationAttr.vector.z);                                        buf.prnLine(ls);
+            data.animationAttr.vector.x,data.animationAttr.vector.y,
+            data.animationAttr.vector.z);					buf.prnLine(ls);
     sprintf(ls,"animation - flashing = %s",
-        (data.flags & trpgLightAttr::trpg_Flashing ? "yes" : "no" ));        buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Flashing ? "yes" : "no" ));	buf.prnLine(ls);
     sprintf(ls,"animation - rotating = %s",
-        (data.flags & trpgLightAttr::trpg_Rotating ? "yes" : "no" ));        buf.prnLine(ls);
+            (data.flags & trpgLightAttr::trpg_Rotating ? "yes" : "no" ));	buf.prnLine(ls);
     sprintf(ls,"animation - counter clockwise = %s",
-        (data.flags & trpgLightAttr::trpg_ClockWise ? "yes" : "no" ));        buf.prnLine(ls);
-    if (data.commentStr) {
+            (data.flags & trpgLightAttr::trpg_ClockWise ? "yes" : "no" ));	buf.prnLine(ls);
+    if (data.commentStr)
+    {
         sprintf(ls,"comment = %s",data.commentStr);  buf.prnLine(ls);
     }
 
@@ -776,19 +855,21 @@ bool trpgLightTable::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Light Table----");
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<lightList.size();i++) {
-        sprintf(ls,"Light %d",i); buf.prnLine(ls);
-        lightList[i].Print(buf);
+    LightMapType::const_iterator itr = lightMap.begin();
+    for (  ; itr != lightMap.end( ); itr++)
+    {
+        sprintf(ls,"Light %d",itr->first); buf.prnLine(ls);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
 
     buf.prnLine();
-
+	
     return true;
 }
 
 /* Print out a light node
-*/
+ */
 
 bool trpgLight::Print(trpgPrintBuffer &buf) const
 {
@@ -797,8 +878,8 @@ bool trpgLight::Print(trpgPrintBuffer &buf) const
     buf.prnLine("----Light----");
     buf.IncreaseIndent();
 
-    sprintf(ls,"Light Index = %d",index);                        buf.prnLine(ls);
-    sprintf(ls,"# Light Locations = %d",lightPoints.size() );    buf.prnLine(ls);
+    sprintf(ls,"Light Index = %d",index);					buf.prnLine(ls);
+    sprintf(ls,"# Light Locations = %d",static_cast<int>(lightPoints.size()) ); buf.prnLine(ls);
 
     buf.DecreaseIndent();
 
@@ -834,9 +915,11 @@ bool trpgRangeTable::Print(trpgPrintBuffer &buf) const
 
     buf.prnLine("----Range Table----");
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<rangeList.size();i++) {
+    RangeMapType::const_iterator itr = rangeMap.begin();
+    for (int i = 0; itr != rangeMap.end( ); itr++, i++)
+    {
         sprintf(ls,"----Range %d----",i);  buf.prnLine(ls);
-        rangeList[i].Print(buf);
+        itr->second.Print(buf);
     }
 
     buf.DecreaseIndent();
@@ -861,10 +944,11 @@ bool trpgLabel::Print(trpgPrintBuffer &buf) const
     sprintf(ls,"desc = %s",desc.c_str());  buf.prnLine(ls);
     sprintf(ls,"url = %s",url.c_str());  buf.prnLine(ls);
     sprintf(ls,"location: (%f %f %f)",location.x,location.y,location.z); buf.prnLine(ls);
-    sprintf(ls,"%d support points",supports.size());  buf.prnLine(ls);
+    sprintf(ls,"%d support points",static_cast<int>(supports.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<supports.size();i++) {
-        sprintf(ls,"%f %f %f",supports[i].x,supports[i].y,supports[i].z); buf.prnLine(ls);    
+    for (unsigned int i=0;i<supports.size();i++)
+    {
+        sprintf(ls,"%f %f %f",supports[i].x,supports[i].y,supports[i].z); buf.prnLine(ls);	
     }
     buf.DecreaseIndent();
     buf.prnLine();
@@ -899,11 +983,13 @@ bool trpgTextStyleTable::Print(trpgPrintBuffer &buf) const
     buf.prnLine();
     buf.prnLine("----Text Style Table----");
     buf.IncreaseIndent();
-    sprintf(ls,"numStyle = %d",styles.size());  buf.prnLine(ls);
+    sprintf(ls,"numStyle = %d",static_cast<int>(styleMap.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<styles.size();i++) {
+    StyleMapType::const_iterator itr = styleMap.begin();
+    for (int i = 0; itr != styleMap.end( ); itr++, i++)
+    {
         sprintf(ls,"Style %d",i);  buf.prnLine(ls);
-        styles[i].Print(buf);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
     buf.DecreaseIndent();
@@ -935,11 +1021,13 @@ bool trpgSupportStyleTable::Print(trpgPrintBuffer &buf) const
     buf.prnLine();
     buf.prnLine("----Support Style Table----");
     buf.IncreaseIndent();
-    sprintf(ls,"numStyle = %d",styles.size());  buf.prnLine(ls);
+    sprintf(ls,"numStyle = %d",static_cast<int>(supportStyleMap.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<styles.size();i++) {
+    SupportStyleMapType::const_iterator itr = supportStyleMap.begin();
+    for (int i = 0; itr != supportStyleMap.end( ); itr++, i++)
+    {
         sprintf(ls,"Style %d",i);  buf.prnLine(ls);
-        styles[i].Print(buf);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
     buf.DecreaseIndent();
@@ -973,11 +1061,13 @@ bool trpgLabelPropertyTable::Print(trpgPrintBuffer &buf) const
     buf.prnLine();
     buf.prnLine("----Label Property Table----");
     buf.IncreaseIndent();
-    sprintf(ls,"numProperty = %d",properties.size());  buf.prnLine(ls);
+    sprintf(ls,"numProperty = %d",static_cast<int>(labelPropertyMap.size()));  buf.prnLine(ls);
     buf.IncreaseIndent();
-    for (unsigned int i=0;i<properties.size();i++) {
+    LabelPropertyMapType::const_iterator itr = labelPropertyMap.begin();
+    for (int i = 0; itr != labelPropertyMap.end( ); itr++, i++)
+    {
         sprintf(ls,"Property %d",i);  buf.prnLine(ls);
-        properties[i].Print(buf);
+        itr->second.Print(buf);
     }
     buf.DecreaseIndent();
     buf.DecreaseIndent();
