@@ -232,6 +232,7 @@ void SoftShadowMap::init()
         // create the camera
         _camera = new osg::Camera;
 
+        _camera->setInheritanceMask(0x0);
         _camera->setCullCallback(new CameraCullCallback(this));
 
         _camera->setClearMask(GL_DEPTH_BUFFER_BIT);
@@ -338,7 +339,7 @@ void SoftShadowMap::cull(osgUtil::CullVisitor& cv)
 
     osgUtil::RenderStage* orig_rs = cv.getRenderStage();
 
-    // do traversal of shadow recieving scene which does need to be decorated by the shadow map
+    // do traversal of shadow receiving scene which does need to be decorated by the shadow map
     {
         cv.pushStateSet(_stateset.get());
 
