@@ -1841,6 +1841,7 @@ void GraphicsWindowWin32::useCursor( bool cursorOn )
 
 void GraphicsWindowWin32::setCursor( MouseCursor mouseCursor )
 {
+    _mouseCursor = mouseCursor;
     HCURSOR newCursor = getOrCreateCursor( mouseCursor);
     if (newCursor == _currentCursor) return;
 
@@ -2134,23 +2135,8 @@ LRESULT GraphicsWindowWin32::handleNativeWindowingEvent( HWND hwnd, UINT uMsg, W
                 RECT clientRect;
                 ::GetClientRect(hwnd, &clientRect);
 
-                int windowWidth;
-                int windowHeight;
-
-                if (clientRect.bottom==0 && clientRect.right==0)
-                {
-                    //
-                    // Window has been minimized; keep window width & height to a minimum of 1 pixel
-                    //
-
-                    windowWidth  = 1;
-                    windowHeight = 1;
-                }
-                else
-                {
-                    windowWidth  = clientRect.right;
-                    windowHeight = clientRect.bottom;
-                }
+                int windowWidth = (clientRect.right == 0) ? 1 : clientRect.right ;
+                int windowHeight = (clientRect.bottom == 0) ? 1 : clientRect.bottom;;
 
                 if (windowX!=_traits->x || windowY!=_traits->y || windowWidth!=_traits->width || windowHeight!=_traits->height)
                 {
@@ -2191,11 +2177,17 @@ LRESULT GraphicsWindowWin32::handleNativeWindowingEvent( HWND hwnd, UINT uMsg, W
         ///////////////////
         case WM_SETCURSOR :
         ///////////////////
-            if (_traits->useCursor) 
-                ::SetCursor( _currentCursor);
-            else
-                ::SetCursor(NULL);
-            return TRUE;
+            //The cursor is only modified in response to the WM_SETCURSOR message if the mouse cursor isn't set to
+            //InheritCursor.  InheritCursor lets the user manage the cursor externally.
+            if (_mouseCursor != InheritCursor)
+            {
+                if (_traits->useCursor) 
+                    ::SetCursor( _currentCursor);
+                else
+                    ::SetCursor(NULL);
+                return TRUE;
+            }
+            break;
 
         /////////////////
         case WM_CLOSE   :
