@@ -30,7 +30,7 @@
 using namespace osgViewer;
 
 //#define DEBUG_MESSAGE osg::notify(osg::NOTICE)
-#define DEBUG_MESSAGE osg::notify(osg::INFO)
+#define DEBUG_MESSAGE osg::notify(osg::DEBUG_FP)
 
 
 OpenGLQuerySupport::OpenGLQuerySupport():
@@ -161,6 +161,10 @@ static OpenThreads::Mutex s_drawSerializerMutex;
 Renderer::Renderer(osg::Camera* camera):
     osg::GraphicsOperation("Renderer",true),
     OpenGLQuerySupport(),
+    _targetFrameRate(100.0),
+    _minimumTimeAvailableForGLCompileAndDeletePerFrame(0.001),
+    _flushTimeRatio(0.5),
+    _conservativeTimeRatio(0.5),
     _camera(camera),
     _done(false),
     _graphicsThreadDoesCull(true)
@@ -254,7 +258,7 @@ void Renderer::cull()
 
     if (_done || _graphicsThreadDoesCull) return;
 
-    // note we assume lock has already been aquired.
+    // note we assume lock has already been acquired.
     osgUtil::SceneView* sceneView = _availableQueue.takeFront();
 
     DEBUG_MESSAGE<<"cull() got SceneView "<<sceneView<<std::endl;
@@ -274,7 +278,7 @@ void Renderer::cull()
         const osg::FrameStamp* fs = state->getFrameStamp();
         int frameNumber = fs ? fs->getFrameNumber() : 0;
 
-        // do cull taversal
+        // do cull traversal
         osg::Timer_t beforeCullTick = osg::Timer::instance()->tick();
 
         sceneView->inheritCullSettings(*(sceneView->getCamera()));
@@ -356,15 +360,15 @@ void Renderer::draw()
             state->getDynamicObjectRenderingCompletedCallback()->completed(state);
         }
 
-        bool aquireGPUStats = stats && _timerQuerySupported && stats->collectStats("gpu");
+        bool acquireGPUStats = stats && _timerQuerySupported && stats->collectStats("gpu");
 
-        if (aquireGPUStats) 
+        if (acquireGPUStats) 
         {
             checkQuery(stats);
         }
 
-        // do draw traveral
-        if (aquireGPUStats) 
+        // do draw traversal
+        if (acquireGPUStats) 
         {
             checkQuery(stats);
             beginQuery(frameNumber);
@@ -389,22 +393,14 @@ void Renderer::draw()
 
         _availableQueue.add(sceneView);
 
-        double availableTime = 0.004; // 4 ms
-        if (databasePager && databasePager->requiresExternalCompileGLObjects(sceneView->getState()->getContextID()))
-        {
-            databasePager->compileGLObjects(*(sceneView->getState()), availableTime);
-        }
+        osg::Timer_t afterDispatchTick = osg::Timer::instance()->tick();
 
-        if (compileThread)
-        {
-            compileThread->add(_flushOperation.get());
-        }
-        else
-        {
-            sceneView->flushDeletedGLObjects(availableTime);
-        }
+        double dispatchTime = osg::Timer::instance()->delta_s(beforeDrawTick, afterDispatchTick);
 
-        if (aquireGPUStats)
+        // now flush delete OpenGL objects and compile any objects as required by the DatabasePager
+        flushAndCompile(dispatchTime, sceneView, databasePager, compileThread);
+    
+        if (acquireGPUStats)
         {
             endQuery();
             checkQuery(stats);
@@ -464,14 +460,14 @@ void Renderer::cull_draw()
         initialize(state);
     }
 
-    bool aquireGPUStats = stats && _timerQuerySupported && stats->collectStats("gpu");
+    bool acquireGPUStats = stats && _timerQuerySupported && stats->collectStats("gpu");
 
-    if (aquireGPUStats) 
+    if (acquireGPUStats) 
     {
         checkQuery(stats);
     }
 
-    // do cull taversal
+    // do cull traversal
     osg::Timer_t beforeCullTick = osg::Timer::instance()->tick();
 
     sceneView->inheritCullSettings(*(sceneView->getCamera()));
@@ -487,8 +483,8 @@ void Renderer::cull_draw()
 #endif
 
 
-    // do draw traveral
-    if (aquireGPUStats) 
+    // do draw traversal
+    if (acquireGPUStats) 
     {
         checkQuery(stats);
         beginQuery(frameNumber);
@@ -511,22 +507,14 @@ void Renderer::cull_draw()
         sceneView->draw();
     }
 
-    double availableTime = 0.004; // 4 ms
-    if (databasePager && databasePager->requiresExternalCompileGLObjects(sceneView->getState()->getContextID()))
-    {
-        databasePager->compileGLObjects(*(sceneView->getState()), availableTime);
-    }
+    osg::Timer_t afterDispatchTick = osg::Timer::instance()->tick();
+    double cullAndDispatchTime = osg::Timer::instance()->delta_s(beforeCullTick, afterDispatchTick);
 
-    if (compileThread)
-    {
-        compileThread->add(_flushOperation.get());
-    }
-    else
-    {
-        sceneView->flushDeletedGLObjects(availableTime);
-    }
+    // now flush delete OpenGL objects and compile any objects as required by the DatabasePager
+    flushAndCompile(cullAndDispatchTime, sceneView, databasePager, compileThread);
 
-    if (aquireGPUStats)
+
+    if (acquireGPUStats)
     {
         endQuery();
         checkQuery(stats);
@@ -551,6 +539,54 @@ void Renderer::cull_draw()
 
 }
 
+void Renderer::flushAndCompile(double currentElapsedFrameTime, osgUtil::SceneView* sceneView, osgDB::DatabasePager* databasePager, osg::GraphicsThread* compileThread)
+{
+    
+    double targetFrameRate = _targetFrameRate;
+    double minimumTimeAvailableForGLCompileAndDeletePerFrame = _minimumTimeAvailableForGLCompileAndDeletePerFrame;
+
+    if (databasePager)
+    {
+        targetFrameRate = std::min(targetFrameRate, databasePager->getTargetFrameRate());
+        minimumTimeAvailableForGLCompileAndDeletePerFrame = std::min(minimumTimeAvailableForGLCompileAndDeletePerFrame, databasePager->getMinimumTimeAvailableForGLCompileAndDeletePerFrame());
+    }
+    
+    double targetFrameTime = 1.0/targetFrameRate;
+
+    double availableTime = std::max((targetFrameTime - currentElapsedFrameTime)*_conservativeTimeRatio,
+                                    minimumTimeAvailableForGLCompileAndDeletePerFrame);
+
+    double flushTime = availableTime * _flushTimeRatio;
+    double compileTime = availableTime - flushTime;
+
+#if 0
+    osg::notify(osg::NOTICE)<<"total availableTime = "<<availableTime*1000.0<<std::endl;
+    osg::notify(osg::NOTICE)<<"      flushTime     = "<<flushTime*1000.0<<std::endl;
+    osg::notify(osg::NOTICE)<<"      compileTime   = "<<compileTime*1000.0<<std::endl;
+#endif
+
+    if (compileThread)
+    {
+        compileThread->add(_flushOperation.get());
+    }
+    else
+    {
+        sceneView->flushDeletedGLObjects(flushTime);
+    }
+
+    // if any time left over from flush add this to compile time.        
+    if (flushTime>0.0) compileTime += flushTime;
+
+#if 0
+    osg::notify(osg::NOTICE)<<"      revised compileTime   = "<<compileTime*1000.0<<std::endl;
+#endif
+
+    if (databasePager && databasePager->requiresExternalCompileGLObjects(sceneView->getState()->getContextID()))
+    {
+        databasePager->compileGLObjects(*(sceneView->getState()), compileTime);
+    }
+}
+
 void Renderer::operator () (osg::Object* object)
 {
     osg::GraphicsContext* context = dynamic_cast<osg::GraphicsContext*>(object);
